@@ -3,7 +3,7 @@
  */
 
 /* Copyright (c) 1995,1996 Sascha Demetrio
- * Copyright (c) 2009, 2010 Peter Pentchev
+ * Copyright (c) 2009, 2010, 2015 Peter Pentchev
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -11,14 +11,14 @@
  * are met:
  * 1. Redistributions of source code must retain the above copyright
  *    notice, this list of conditions and the following disclaimer.
- *    If you modify any part of HEXER and resitribute it, you must add
+ *    If you modify any part of HEXER and redistribute it, you must add
  *    a notice to the `README' file and the modified source files containing
  *    information about the  changes you made.  I do not want to take
  *    credit or be blamed for your modifications.
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- *    If you modify any part of HEXER and resitribute it in binary form,
+ *    If you modify any part of HEXER and redistribute it in binary form,
  *    you must supply a `README' file containing information about the
  *    changes you made.
  * 3. The name of the developer may not be used to endorse or promote
@@ -57,6 +57,7 @@
 
 #include "buffer.h"
 #include "defs.h"
+#include "util.h"
 
 struct BufferOptions b_default_options = {
   BUFFER_BLOCKSIZE,
@@ -84,7 +85,7 @@ Buffer *last_buffer = 0;
 
 #if !HAVE_MEMMOVE
   static void
-b_memmove(char *t, const char *s, const long count)
+b_memmove(char *t, const char *s, const unsigned long count)
 {
   register long i;
 
@@ -107,13 +108,11 @@ b_memmove(char *t, const char *s, const long count)
     last_position = 0, last_number = 1, last_length = -1;                     \
   buffer->modified = 1; }
 
-  long
-count_lines(source, count)
-  char *source;
-  long count;
+  unsigned long
+count_lines(char *source, unsigned long count)
 {
   char *i;
-  long j = 0;
+  unsigned long j = 0;
 
   for (i = source; i < source + count; i++) if (*i == '\n') j++;
   return j;
@@ -125,27 +124,21 @@ count_lines(source, count)
  */
 
   BufferBlock *
-new_buffer_block(blocksize, data)
-  unsigned long blocksize;
-  char *data;
+new_buffer_block(const unsigned long blocksize, char * const data)
 {
   BufferBlock *new_block;
 
-  if (!(new_block = (BufferBlock *)malloc(sizeof(BufferBlock)))) return 0;
+  new_block = (BufferBlock *)malloc_fatal(sizeof(BufferBlock));
   new_block->next_block = 0;
 
   if (data != 0) new_block->data = data;
-  else if (!(new_block->data = (char *)malloc(blocksize))) {
-    free((char *)new_block);
-    return 0;
-  }
+  else new_block->data = (char *)malloc_fatal(blocksize);
   return new_block;
 }
 /* new_buffer_block */
 
   void
-delete_buffer_block(block)
-  BufferBlock *block;
+delete_buffer_block(BufferBlock * const block)
 {
   free((char *)block->data);
   free((char *)block);
@@ -157,14 +150,13 @@ delete_buffer_block(block)
  */
 
   Buffer *
-new_buffer(arg_options)
-  struct BufferOptions *arg_options;
+new_buffer(struct BufferOptions *arg_options)
 {
   Buffer * new_buf;
   struct BufferOptions options;
   
   options = arg_options ? *arg_options : b_default_options;
-  if (!(new_buf = (Buffer *)malloc(sizeof(Buffer)))) return 0;
+  new_buf = (Buffer *)malloc_fatal(sizeof(Buffer));
   new_buf->first_block = 0;
   new_buf->size = 0;
   new_buf->blocksize = options.blocksize;
@@ -175,8 +167,7 @@ new_buffer(arg_options)
 /* new_buffer */
 
   int
-delete_buffer(buffer)
-  Buffer *buffer;
+delete_buffer(Buffer * const buffer)
 {
   BufferBlock *i, *j;
 
@@ -189,41 +180,11 @@ delete_buffer(buffer)
 }
 /* delete_buffer */
 
-  int
-copy_buffer(target, source)
-  Buffer *target;
-  Buffer *source;
-  /* NOTE: `copy_buffer()' only copies the contents of `source', i.e.
-   *   the blacksize of `target' is kept unchanged.
-   */
-{
-  BufferBlock *i;
-  unsigned long position = 0;
-  
-  assert(!target->read_only);
-  if (target == source) return 0;
-  b_clear(target); 
-  for (i = source->first_block; i; i = i->next_block)
-    position += b_write(target, i->data, position, source->blocksize);
-  return 0;
-}
-/* copy_buffer */
-
-  void
-clear_buffer(buffer)
-  Buffer *buffer;
-{
-  b_clear(buffer);
-}
-/* clear_buffer */
-
   BufferBlock *
-find_block(buffer, position)
-  Buffer *buffer;
-  unsigned long position;
+find_block(Buffer *buffer, unsigned long position)
 {
   BufferBlock *i;
-  long block_number;
+  unsigned long block_number;
 
   if (position >= buffer->size) return 0;
   for (i = buffer->first_block, block_number = 1;
@@ -235,12 +196,10 @@ find_block(buffer, position)
 /* find_block */
 
   int
-b_set_size(buffer, size)
-  Buffer *buffer;
-  unsigned long size;
+b_set_size(Buffer *buffer, unsigned long size)
 {
   BufferBlock *i, *j = 0, *k;
-  long block_number;
+  unsigned long block_number;
 
   assert(!buffer->read_only);
   BUFFER_CHANGED(buffer);
@@ -272,35 +231,30 @@ b_set_size(buffer, size)
 /* b_set_size */
 
   long
-b_read(buffer, target, position, count)
-  Buffer *buffer;
-  char *target;
-  long position;
-  long count;
+b_read(Buffer *buffer, char *target, unsigned long position, unsigned long count)
 {
   BufferBlock *block;
   unsigned long bs = buffer->blocksize;
+  unsigned long ofs = position % bs;
+  unsigned long r = bs - ofs;
 
-  if (!(block = find_block(buffer, position))) return 0;
+  if (!(block = find_block(buffer, position))) return -1;
   if (count + position > buffer->size) count = buffer->size - position;
-  if (count <= (bs - position % bs)) {
-    b_memmove(target, block->data + position % bs, count);
+  if (count <= r) {
+    b_memmove(target, block->data + ofs, count);
     return count;
   } else {
-    long r = bs - position % bs;
-    b_memmove(target, block->data + position % bs, r);
-    r += b_read(buffer, target + r, position + r, count - r);
-    return r;
+    b_memmove(target, block->data + ofs, r);
+    long more = b_read(buffer, target + r, position + r, count - r);
+    if (more < 0)
+      return more;
+    return r + more;
   }
 }
 /* b_read */
 
-  long
-b_write(buffer, source, position, count)
-  Buffer *buffer;
-  char *source;
-  long position;
-  long count;
+  unsigned long
+b_write(Buffer *buffer, char *source, unsigned long position, unsigned long count)
 {
   BufferBlock *block;
   unsigned long bs = buffer->blocksize;
@@ -313,7 +267,7 @@ b_write(buffer, source, position, count)
     b_memmove(block->data + position % bs, source, count);
     return count;
   } else {
-    long r = bs - position % bs;
+    unsigned long r = bs - position % bs;
     b_memmove(block->data + position % bs, source, r);
     return r + b_write(buffer, source + r, position + r, count - r);
   }
@@ -321,11 +275,7 @@ b_write(buffer, source, position, count)
 /* b_write */
 
   long
-b_write_append(buffer, source, position, count)
-  Buffer *buffer;
-  char *source;
-  long position;
-  long count;
+b_write_append(Buffer *buffer, char *source, unsigned long position, unsigned long count)
 {
   assert(!buffer->read_only);
   if (position + count > buffer->size) {
@@ -336,58 +286,14 @@ b_write_append(buffer, source, position, count)
 /* b_write_append */
 
   long
-b_append(buffer, source, count)
-  Buffer *buffer;
-  char *source;
-  long count;
+b_append(Buffer *buffer, char *source, unsigned long count)
 {
   return b_write_append(buffer, source, buffer->size, count);
 }
 /* b_append */
 
-  long
-b_fill(buffer, c, position, count)
-  Buffer *buffer;
-  char c;
-  long position;
-  long count;
-{
-  BufferBlock *block;
-  unsigned long bs = buffer->blocksize;
-
-  assert(!buffer->read_only);
-  if (count) BUFFER_CHANGED(buffer);
-  if (!(block = find_block(buffer, position))) return 0;
-  if (count + position > buffer->size) count = buffer->size - position;
-  if (count <= (bs - position % bs)) {
-    memset(block->data + position % bs, c, count);
-    return count;
-  } else {
-    long r = bs - position % bs;
-    memset(block->data + position % bs, c, r);
-    return r + b_fill(buffer, c, position + r, count - r);
-  }
-}
-/* b_fill */
-
-  long
-b_fill_append(buffer, c, position, count)
-  Buffer *buffer;
-  char c;
-  long position;
-  long count;
-{
-  if (position + count > buffer->size)
-    b_set_size(buffer, position + count);
-  return b_fill(buffer, c, position, count);
-}
-/* b_fill_append */
-
-  long
-b_count_lines(buffer, position, count)
-  Buffer *buffer;
-  long position;
-  long count;
+  unsigned long
+b_count_lines(Buffer *buffer, unsigned long position, unsigned long count)
 {
   BufferBlock *block;
   unsigned long bs = buffer->blocksize;
@@ -397,7 +303,7 @@ b_count_lines(buffer, position, count)
   if (count <= bs - position % bs) 
     return count_lines(block->data + position % bs, count);
   else {
-    long r = bs - position % bs;
+    unsigned long r = bs - position % bs;
     return count_lines(block->data + position % bs, r)
             + b_count_lines(buffer, position + r, count - r);
   }
@@ -405,12 +311,9 @@ b_count_lines(buffer, position, count)
 /* b_count_lines */
 
   long
-b_insert(buffer, position, count)
-  Buffer *buffer;
-  long position;
-  long count;
+b_insert(Buffer *buffer, unsigned long position, unsigned long count)
 {
-  long i;
+  unsigned long i;
 
   assert(!buffer->read_only);
   if(b_set_size(buffer, buffer->size + count) < 0) return -1;
@@ -421,32 +324,25 @@ b_insert(buffer, position, count)
 /* b_insert */
 
   long
-b_delete(buffer, position, count)
-  Buffer *buffer;
-  long position;
-  long count;
+b_delete(Buffer *buffer, unsigned long position, unsigned long count)
 {
-  long size = buffer->size;
+  unsigned long size = buffer->size;
 
   assert(!buffer->read_only);
-  b_copy(buffer, buffer, position, position + count,
-         buffer->size - position - count);
+  if (b_copy(buffer, buffer, position, position + count,
+         buffer->size - position - count) < 0)
+    return -1;
   if (b_set_size(buffer, buffer->size - count) < 0) return -1;
   return size - buffer->size;
 }
 /* b_delete */
 
   long
-b_copy(target_buffer, source_buffer, target_position, source_position, count)
-  Buffer *target_buffer;
-  Buffer *source_buffer;
-  long target_position;
-  long source_position;
-  long count;
+b_copy(Buffer *target_buffer, Buffer *source_buffer, unsigned long target_position, unsigned long source_position, unsigned long count)
 {
   BufferBlock *t_block;
-  long t_offset = target_position % target_buffer->blocksize;
-  long t_blocksize = target_buffer->blocksize;
+  unsigned long t_offset = target_position % target_buffer->blocksize;
+  unsigned long t_blocksize = target_buffer->blocksize;
 
   assert(!target_buffer->read_only);
   if (count) BUFFER_CHANGED(target_buffer);
@@ -465,28 +361,30 @@ b_copy(target_buffer, source_buffer, target_position, source_position, count)
   } else {
     long r = b_read(source_buffer, t_block->data + t_offset,
                     source_position, t_blocksize - t_offset);
-    if (r >= t_blocksize - t_offset)
-      r += b_copy(target_buffer, source_buffer,
+    if (r > 0 && (unsigned long)r >= t_blocksize - t_offset)
+    {
+      long more = b_copy(target_buffer, source_buffer,
                   target_position + r, source_position + r, count - r);
+      if (more < 0)
+	return more;
+      else
+	r += more;
+    }
     return r;
   }
 }
 /* b_copy */
 
   long
-b_copy_forward(buffer, target_position, source_position, count)
-  Buffer *buffer;
-  long target_position;
-  long source_position;
-  long count;
+b_copy_forward(Buffer *buffer, unsigned long target_position, unsigned long source_position, unsigned long count)
 {
   BufferBlock *t_block, *s_block;
-  long bs = buffer->blocksize;
-  long t_offset = target_position % bs; /* target offset. */
-  long s_offset = source_position % bs; /* source offset. */
-  long tr_offset = (target_position + count - 1) % bs + 1;
+  unsigned long bs = buffer->blocksize;
+  unsigned long t_offset = target_position % bs; /* target offset. */
+  unsigned long s_offset = source_position % bs; /* source offset. */
+  unsigned long tr_offset = (target_position + count - 1) % bs + 1;
     /* target reverse offset. */
-  long sr_offset = (source_position + count) % bs;
+  unsigned long sr_offset = (source_position + count) % bs;
     /* source reverse offset. */
 
   assert(!buffer->read_only);
@@ -511,7 +409,7 @@ b_copy_forward(buffer, target_position, source_position, count)
     }
     return count;
   } else {
-    long r = tr_offset;
+    unsigned long r = tr_offset;
     if (tr_offset >= sr_offset) {
       if (sr_offset) {
         assert(s_block = find_block(buffer, source_position + count));
@@ -536,8 +434,7 @@ b_copy_forward(buffer, target_position, source_position, count)
 /* b_copy_forward */
 
   void
-b_clear(buffer)
-  Buffer *buffer;
+b_clear(Buffer * const buffer)
 {
   BufferBlock *i, *j;
 
@@ -553,14 +450,12 @@ b_clear(buffer)
 /* b_clear */
 
   long
-b_read_buffer_from_file(buffer, filename)
-  Buffer *buffer;
-  char *filename;
+b_read_buffer_from_file(Buffer * const buffer, const char * const filename)
 {
   BufferBlock *i = 0;
-  long bytes_read = 0;
   char *tmp;
   int file = open(filename, O_RDONLY);
+  int err_f = 0;
     
   assert(!buffer->read_only);
   BUFFER_CHANGED(buffer);
@@ -570,11 +465,9 @@ b_read_buffer_from_file(buffer, filename)
   }
   b_clear(buffer);
   do {
-    tmp = (char *)malloc(buffer->blocksize);
-    if (tmp == 0) {
-      close(file);
-      return -1;
-    }
+    ssize_t bytes_read;
+
+    tmp = (char *)malloc_fatal(buffer->blocksize);
     bytes_read = read(file, tmp, buffer->blocksize);
     if (bytes_read > 0)	{
       if (buffer->first_block == 0) {
@@ -587,31 +480,36 @@ b_read_buffer_from_file(buffer, filename)
       }
       buffer->size += bytes_read;
     } else {
+      err_f = bytes_read < 0;
       free((char *)tmp);
       break;
     }
   } while (1);
   close(file);
+  if (err_f) {
+    b_clear(buffer);
+    return -1;
+  }
   return buffer->size;
 }
 /* b_read_buffer_from_file */
 
   long
-b_write_buffer_to_file(buffer, filename)
-  Buffer *buffer;
-  char *filename;
+b_write_buffer_to_file(Buffer *buffer, char *filename)
 {
   BufferBlock *i;
-  long bytes_wrote = 0;
-  long bs = buffer->blocksize;
-  long blocks;
+  unsigned long bytes_wrote = 0;
+  unsigned long bs = buffer->blocksize;
+  unsigned long blocks;
   int file = open(filename, O_WRONLY | O_CREAT | O_TRUNC, 0666);
 
+  if (file < 0)
+    return -1;
   for (i = buffer->first_block, blocks = 1;
        i;
        i = i->next_block, blocks++) {
-    long bytes;
-    bytes = write(file,
+    ssize_t bytes;
+    bytes = write_buf(file,
                   i->data,
                   (blocks * bs > buffer->size) ? buffer->size % bs : bs);
     if (bytes < 0) {
@@ -626,18 +524,14 @@ b_write_buffer_to_file(buffer, filename)
 /* b_write_buffer_to_file */
 
   long
-b_copy_to_file(buffer, filename, position, count)
-  Buffer *buffer;
-  char *filename;
-  long position;
-  long count;
+b_copy_to_file(Buffer *buffer, const char *filename, unsigned long position, unsigned long count)
 {
-  char *tmp = malloc(buffer->blocksize);
+  char *tmp = malloc_fatal(buffer->blocksize);
   long bytes_read = 0, bytes_wrote = 0;
   int file = open(filename, O_WRONLY | O_CREAT | O_TRUNC, 0666);
 
-  if (file < 0 || !tmp) {
-    if (tmp) free((char *)tmp);
+  if (file < 0) {
+    free((char *)tmp);
     if (file >= 0) close(file);
     return -1;
   }
@@ -645,7 +539,7 @@ b_copy_to_file(buffer, filename, position, count)
     bytes_read =
       b_read(buffer, tmp, position,
 	     (buffer->blocksize < count) ? buffer->blocksize : count);
-    if (write(file, tmp, bytes_read) < 0) {
+    if (bytes_read < 0 || write_buf(file, tmp, bytes_read) < 0) {
       free((char *)tmp);
       close(file);
       return -1;
@@ -662,25 +556,22 @@ b_copy_to_file(buffer, filename, position, count)
 /* b_copy_to_file */
 
   long
-b_paste_from_file(buffer, filename, position)
-  Buffer *buffer;
-  char *filename;
-  long position;
+b_paste_from_file(Buffer *buffer, const char *filename, unsigned long position)
 {
-  char *tmp = malloc(buffer->blocksize);
+  char *tmp = malloc_fatal(buffer->blocksize);
   long bytes_read, bytes_wrote = 0;
   int file = open(filename, O_RDONLY);
   
   assert(!buffer->read_only);
-  if (file < 0 || !tmp) {
-    if (tmp) free((char *)tmp);
+  if (file < 0) {
+    free((char *)tmp);
     if (file >= 0) close(file);
     return -1;
   }
   BUFFER_CHANGED(buffer);
   do {
     bytes_read = read(file, tmp, buffer->blocksize);
-    if (b_insert(buffer, position, bytes_read) < 0) {
+    if (bytes_read < 0 || b_insert(buffer, position, bytes_read) < 0) {
       free((char *)tmp);
       close(file);
       return -1;
@@ -698,31 +589,28 @@ b_paste_from_file(buffer, filename, position)
 /* The following set of functions is for dealing with text-buffers.
  */
 
-  long
-b_no_lines(buffer)
-  Buffer *buffer;
+  unsigned long
+b_no_lines(Buffer *buffer)
 {
   return b_count_lines(buffer, 0, buffer -> size);
 }
 /* b_no_lines */
 
   long
-b_goto_line(buffer, number)
-  Buffer *buffer;
-  long number;
+b_goto_line(Buffer *buffer, unsigned long number)
 {
   BufferBlock *i;
-  long newlines = 0, blocks = 0;
-  char *c;
+  unsigned long newlines = 0, blocks = 0;
 
   if (number == 1) return 0;
-  if (buffer == last_buffer && number == last_number) return last_position;
+  if (buffer == last_buffer && last_number >= 0 && number == (unsigned long)last_number) return last_position;
   for (i = buffer -> first_block; i; i = i -> next_block) {
-    for (c = i -> data; c < i -> data + buffer -> blocksize; c++)
-      if (*c == '\n')
+    size_t ofs;
+    for (ofs = 0; ofs < buffer -> blocksize; ofs++)
+      if (i->data[ofs] == '\n')
         if (++newlines == number - 1) {
-          long block_position = blocks * buffer -> blocksize;
-          last_position = block_position + (long)c - (long)(i -> data) + 1;
+          unsigned long block_position = blocks * buffer -> blocksize;
+          last_position = block_position + ofs + 1;
           last_number = number;
           last_buffer = buffer;
           last_length = -1;
@@ -734,185 +622,6 @@ b_goto_line(buffer, number)
 }
 /* b_goto_line */
 
-  long
-b_get_linenumber(buffer, position)
-  Buffer *buffer;
-  long position;
-{
-  long number;
-
-  if (buffer == last_buffer && position == last_position)
-    return last_number;
-  number = b_count_lines(buffer, 0, position - 1) - 1;
-  if (number > 0) {
-    last_number = number;
-    last_position = position;
-    last_length = -1;
-    last_buffer = buffer;
-  }
-  return number;
-}
-/* b_get_linenumber */
-
-  long
-b_line_start(buffer, position)
-  Buffer *buffer;
-  long position;
-{
-  const long bs = buffer -> blocksize;
-  BufferBlock *i;
-  char *c;
-
-  if (position < 0) return 0;
-  if (!(i = find_block(buffer, position))) return -1;
-  for (c = i -> data + position; c >= i -> data; c--)
-    if (*c == '\n')
-      return (position / bs) * bs + (long)c - (long)(i -> data) + 1;
-  return b_line_start(buffer, (position / bs) * bs - 1);
-}
-/* b_line_start */
-
-  long
-b_line_end(buffer, position)
-  Buffer *buffer;
-  long position;
-{
-  const long bs = buffer -> blocksize;
-  BufferBlock *i;
-  char *c;
-
-  if (position >= buffer -> size) return buffer -> size;
-  if (!(i = find_block(buffer, position))) return -1;
-  for (c = i -> data + position % bs; c < i -> data + bs; c++)
-    if (*c == '\n')
-      return (position / bs) * bs + (long)c - (long)(i -> data);
-  return b_line_end(buffer, (position / bs + 1) * bs);
-}
-/* b_line_end */
-
-  long
-b_length_of_line(buffer, number)
-  Buffer *buffer;
-  long number;
-{
-  long position, length;
-
-  if (buffer == last_buffer && number == last_number && last_length != -1)
-    return last_length;
-  position = b_goto_line(buffer, number);
-  length = b_line_end(buffer, position) - position + 1;
-  return last_length = length;
-    /* The values of `last_position' and `last_number' have been set already
-     * by `b_goto_line()'. */
-}
-/* b_length_of_line */
-
-  long
-b_length_of_text_block(buffer, number, count)
-  Buffer *buffer;
-  long number;
-  long count;
-{
-  long start, end;
-
-  start = b_goto_line(buffer, number);
-  end = start;
-  for (; count > 0 && end != buffer -> size; count--) {
-    end = b_line_end(buffer, end) + 1;
-  }
-  return end - start;
-}
-/* b_length_of_text_block */
-
-  long
-b_read_line(buffer, line, number)
-  Buffer *buffer;
-  char *line;
-  long number;
-{
-  long position = b_goto_line(buffer, number);
-  long length = b_length_of_line(buffer, number);
-  long bytes_read = b_read(buffer, line, position, length);
-
-  if (bytes_read < length) bytes_read++;
-  line[bytes_read - 1] = '\0';
-  return bytes_read; 
-}
-/* b_read_line */
-
-  long
-b_read_text_block(buffer, target, number, count)
-  Buffer *buffer;
-  char *target;
-  long number;
-  long count;
-{
-  long start = b_goto_line(buffer, number);
-  long end = start;
-  long bytes_read;
-
-  bytes_read = b_read(buffer, target, start,
-                      b_length_of_text_block(buffer, number, count));
-  if (bytes_read < end - start) bytes_read++;
-  target[bytes_read - 1] = '\0';
-  return bytes_read;
-}
-/* b_read_text_block */
-
-  long
-b_delete_line(buffer, number)
-  Buffer *buffer;
-  long number;
-{
-  long position = b_goto_line(buffer, number);
-  long length = b_length_of_line(buffer, number);
-
-  return b_delete(buffer, position, length);
-}
-/* b_delete_line */
-  
-  long
-b_delete_text_block(buffer, number, count)
-  Buffer *buffer;
-  long number;
-  long count;
-{
-  long start = b_goto_line(buffer, number);
-  long end = start;
-
-  for (; count > 0; count--) end = b_line_end(buffer, end) + 1;
-  return b_delete(buffer, start, end - start);
-}
-/* b_delete_text_block */
-
-  long
-b_clear_line(buffer, number)
-  Buffer *buffer;
-  long number;
-{
-  long position = b_goto_line(buffer, number);
-  long length = b_length_of_line(buffer, number);
-
-  return b_delete(buffer, position, length ? length - 1 : 0);
-}
-/* b_clear_line */
-
-  long
-b_insert_text_block(buffer, source, number)
-  Buffer *buffer;
-  char *source;
-  long number;
-{
-  long position, bytes_inserted;
-
-  bytes_inserted = b_insert(buffer, position = b_goto_line(buffer, number),
-                            strlen(source) + 1);
-  if (bytes_inserted < 0) return -1;
-  source[bytes_inserted - 1] = '\n';
-  return b_write(buffer, source, position, bytes_inserted);
-}
-/* b_insert_text_block */
-
 /* end of buffer.c */
 
 
