@@ -3,7 +3,7 @@
  */
 
 /* Copyright (c) 1995,1996 Sascha Demetrio
- * Copyright (c) 2009, 2010 Peter Pentchev
+ * Copyright (c) 2009, 2010, 2015 Peter Pentchev
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -11,14 +11,14 @@
  * are met:
  * 1. Redistributions of source code must retain the above copyright
  *    notice, this list of conditions and the following disclaimer.
- *    If you modify any part of HEXER and resitribute it, you must add
+ *    If you modify any part of HEXER and redistribute it, you must add
  *    a notice to the `README' file and the modified source files containing
  *    information about the  changes you made.  I do not want to take
  *    credit or be blamed for your modifications.
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- *    If you modify any part of HEXER and resitribute it in binary form,
+ *    If you modify any part of HEXER and redistribute it in binary form,
  *    you must supply a `README' file containing information about the
  *    changes you made.
  * 3. The name of the developer may not be used to endorse or promote
@@ -49,6 +49,7 @@
 
 #define NDEBUG
 
+#include <alloca.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -65,6 +66,7 @@
 #include "calc.h"
 #include "readline.h"
 #include "tio.h"
+#include "util.h"
 
 #define CALC_STACKSIZE 1024
 
@@ -96,7 +98,7 @@ enum calc_object_e {
                      || x.type == CO_FLOAT                                    \
                      || x.type == CO_BOOLEAN)
 
-static char *calc_object_names[] = {
+static const char *calc_object_names[] = {
   "none", "integer", "float", "boolean", "unary operator", "binary operator",
   "variable", "error", 0
 };
@@ -108,7 +110,7 @@ enum calc_unary_operator_e {
   OP_max_unary_operator
 };
 
-static char *calc_unary_operator_names[OP_max_unary_operator] = {
+static const char *calc_unary_operator_names[OP_max_unary_operator] = {
   "-  neg", "!  not", "~  comp", "(  paropen", ")  parclose", ":  exec"
 };
 
@@ -129,7 +131,7 @@ static int prec[OP_max_binary_operator] = {
   0, 1, 1, 1, 2, 2, 3, 3, 4, 4, 4, 4, 5, 5, 6, 6, 6, 7, 7, 8
 };
 
-static char *calc_binary_operator_names[OP_max_binary_operator] = {
+static const char *calc_binary_operator_names[OP_max_binary_operator] = {
   "** pow",
   "*  mul", "/  div", "%  mod", "+  add", "-  sub", "<< shl", ">> shr",
   "<  les", "<= leq", ">  grt", ">= geq", "== equ", "!= neq",
@@ -295,27 +297,16 @@ calc_sprint(char *s, struct calc_object_s object)
 /* calc_print */
 
   static int
-#if USE_STDARG
 calc_error(struct calc_object_s *x, const char *fmt, ...)
-#else
-calc_error(x, fmt, va_alist)
-  struct calc_object_s *x;
-  char *fmt;
-  va_dcl
-#endif
 {
   va_list ap;
   char buf[256];
 
-#if USE_STDARG
   va_start(ap, fmt);
-#else
-  va_start(ap);
-#endif
   vsprintf(buf, fmt, ap);
   if (x->type != CO_ERROR) {
     x->type = CO_ERROR;
-    x->u.s = strdup(buf);
+    x->u.s = strdup_fatal(buf);
   }
   va_end(ap);
   return 0;
@@ -351,9 +342,9 @@ calc_assign(struct calc_object_s x, struct calc_object_s y)
       i->object = y;
       return 0;
     }
-  i = (struct calc_variable_s *)malloc(sizeof *i);
+  i = (struct calc_variable_s *)malloc_fatal(sizeof *i);
   i->object = y;
-  i->name = strdup(x.u.s);
+  i->name = strdup_fatal(x.u.s);
   i->next = calc_variable_list;
   calc_variable_list = i;
   return 0;
@@ -939,7 +930,7 @@ exec: z = stack[sp - 3];
 exit_calc_reduce:
   return error_f ? -1 : reduce_f;
 
-eval: /* evaluate a parenthesed expression */
+eval: /* evaluate a parenthesized expression */
   if (sp > 1 && y.type == CO_UNARY_OPERATOR ? y.u.uo == OP_PAROPEN : 0) {
     calc_clear_stack(2);
     calc_error(stack + sp++, "empty parentheses");
@@ -971,7 +962,7 @@ eval: /* evaluate a parenthesed expression */
   static int
 istrue(char *s, char **endptr)
 {
-  static char *true_strings[] = { "true", "t", "TRUE", "True", "T",
+  static const char *true_strings[] = { "true", "t", "TRUE", "True", "T",
     "yes", "y", "YES", "Yes", "on", "ON", "On", 0 };
   int i, l;
 
@@ -988,7 +979,7 @@ istrue(char *s, char **endptr)
   static int
 isfalse(char *s, char **endptr)
 {
-  static char *false_strings[] = { "false", "f", "FALSE", "False", "F",
+  static const char *false_strings[] = { "false", "f", "FALSE", "False", "F",
     "no", "n", "NO", "No", "off", "OFF", "Off", 0 };
   int i, l;
 
@@ -1052,7 +1043,7 @@ dec:    base = 10, r = p;
     } else if (isalpha(*p) || *p == '_' || *p == '$') {
       for (q = p + 1; isalnum(*q) || *q == '_' || *q == '$'; ++q);
       x.type = CO_VARIABLE;
-      x.u.s = strdup(p);
+      x.u.s = strdup_fatal(p);
       x.u.s[q - p] = 0;
       p = q;
     } else {
@@ -1195,7 +1186,7 @@ dec:    base = 10, r = p;
         x.u.uo = OP_PARCLOSE;
         break;
       default:
-        calc_error(stack + sp++, "unexpected caracter %c (0x%x)",
+        calc_error(stack + sp++, "unexpected character %c (0x%x)",
                    isprint(*p) ? *p : ' ', (int)(unsigned char)*p);
         return -1;
       }
@@ -1229,9 +1220,7 @@ dec:    base = 10, r = p;
 
 #ifdef MYCALC
   int
-main(argc, argv)
-  int argc;
-  char **argv;
+main(int argc, char **argv)
 {
   char buf[256];
   int error_f = 0;
