@@ -2,7 +2,7 @@
  */
 
 /* Copyright (c) 1995,1996 Sascha Demetrio
- * Copyright (c) 2009, 2010 Peter Pentchev
+ * Copyright (c) 2009, 2010, 2015 Peter Pentchev
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -10,14 +10,14 @@
  * are met:
  * 1. Redistributions of source code must retain the above copyright
  *    notice, this list of conditions and the following disclaimer.
- *    If you modify any part of HEXER and resitribute it, you must add
+ *    If you modify any part of HEXER and redistribute it, you must add
  *    a notice to the `README' file and the modified source files containing
  *    information about the  changes you made.  I do not want to take
  *    credit or be blamed for your modifications.
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- *    If you modify any part of HEXER and resitribute it in binary form,
+ *    If you modify any part of HEXER and redistribute it in binary form,
  *    you must supply a `README' file containing information about the
  *    changes you made.
  * 3. The name of the developer may not be used to endorse or promote
@@ -45,6 +45,7 @@
 
 #include "config.h"
 
+#include <alloca.h>
 #include <assert.h>
 #include <pwd.h>
 #include <string.h>
@@ -93,16 +94,17 @@ press_any_key(struct he_s *hedit)
  * null-pointer is returned.
  */
 
-  static char *
-exh_skipcmd(char *cmd, char *dest)
+  static const char *
+exh_skipcmd(const char *cmd, char *dest)
   /* read `cmd' up to the next semicolon.  `\;' escapes the semicolon.
-   * The return value is the character following the first occurence of an
+   * The return value is the character following the first occurrence of an
    * unmasked semicolon or 0 if no semicolon was found or the first unmasked
    * semicolon was the last character in the string `cmd'.  the string read is
    * written to `dest'.
    */
 {
-  char *p, *q;
+  const char *p;
+  char *q;
 
   if (!dest) dest = (char *)alloca(strlen(cmd) + 1);
   for (p = cmd, q = dest; *p && *p != ';';)
@@ -116,11 +118,12 @@ exh_skipcmd(char *cmd, char *dest)
 }
 /* exh_skipcmd */
 
-  static char *
-exh_map(struct he_s *hedit, char *args, int map)
+  static const char *
+exh_map(struct he_s *hedit, const char *args, int map)
 {
   char *from, *to;
-  char *p, *skip;
+  char *p;
+  const char *skip;
 
   skip = exh_skipcmd(args, p = (char *)alloca(strlen(args) + 1));
   args = p;
@@ -136,7 +139,7 @@ exh_map(struct he_s *hedit, char *args, int map)
     free((char *)list);
     return skip;
   }
-  for (p = from = args; *p && *p != ' '; ++p);
+  for (from = p; *p && *p != ' '; ++p);
   if (*p) {
     *p++ = 0;
     while (*p == ' ') ++p;
@@ -148,12 +151,13 @@ exh_map(struct he_s *hedit, char *args, int map)
 }
 /* exh_map */
 
-  static char *
-exh_unmap(struct he_s *hedit, char *args, int map)
+  static const char *
+exh_unmap(struct he_s *hedit __unused, const char *args, int map)
   /* ARGSUSED */
 {
   char *from;
-  char *p, *skip;
+  char *p;
+  const char *skip;
 
   skip = exh_skipcmd(args, p = (char *)alloca(strlen(args) + 1));
   args = p;
@@ -161,27 +165,24 @@ exh_unmap(struct he_s *hedit, char *args, int map)
     he_message(0, "unmap what?");
     return 0;
   }
-  for (p = from = args; *p && *p != ' '; ++p);
+  for (from = p; *p && *p != ' '; ++p);
   *p = 0;
   if (tio_unmap(map, from, he_map_special)) he_message(0, "no such mapping");
   return skip;
 }
 /* exh_unmap */
 
-  static char *
-exh_save_buffer(struct he_s *hedit, char *name, long begin, long end, int *error)
+  static const char *
+exh_save_buffer(struct he_s *hedit, char *name)
 {
   struct buffer_s *i;
   char *p;
-  char *skip;
+  const char *skip;
   int force_f = 0;
   FILE *fp;
   long l;
-  int whole_f = 0;
   char *file = 0;
 
-  if (end >= hedit->buffer->size || end < 0) end = hedit->buffer->size - 1;
-  if (begin == 0 && end == hedit->buffer->size - 1) whole_f = 1;
   while (*name == ' ' || *name == '\t') ++name;
   if (*name == '!') {
     force_f = 1;
@@ -201,40 +202,43 @@ exh_save_buffer(struct he_s *hedit, char *name, long begin, long end, int *error
       }
     if (!i) {
       he_message(0, "@Abno buffer named@~ `%s'", name);
-      if (error) *error = 1;
       return skip;
     }
   }
   if (i->hedit->read_only && !force_f) {
     he_message(0, "@Abread only buffer@~ - use @U:w!@~ to override");
-    if (error) *error = 1;
     return skip;
   }
   if (!(fp = fopen(file, "w"))) {
     he_message(0, "@Abcan't open@~ `%s'", file);
-    if (error) *error = 1;
     return skip;
   }
   fclose(fp);
-  l = b_copy_to_file(i->hedit->buffer, file, begin, end - begin + 1);
-  he_message(0, "wrote `%s' - 0x%lx (%li) bytes", file, l, l);
-  if (whole_f) i->hedit->buffer->modified = 0;
-  if (error) *error = 0;
+  l = b_copy_to_file(i->hedit->buffer, file, 0, hedit->buffer->size);
+  if (l > 0) {
+    he_message(0, "wrote `%s' - 0x%lx (%li) bytes", file, l, l);
+    i->hedit->buffer->modified = 0;
+  } else {
+    he_message(1, "could not write `%s'", file);
+  }
   return skip;
 }
 /* exh_save_buffer */
 
-  static char *
-exh_write(struct he_s *hedit, char *file, long begin, long end, int *error)
+  static const char *
+exh_write(struct he_s *hedit, const char *file, long s_begin, long s_end, int *error)
 {
   char *p;
-  char *skip;
+  const char *skip;
   int force_f = 0;
   FILE *fp;
   long l;
   int whole_f = 0;
 
-  if (end >= hedit->buffer->size || end < 0) end = hedit->buffer->size - 1;
+  unsigned long begin = (unsigned long)s_begin;
+  unsigned long end = (s_end >= 0 && (unsigned long)s_end < hedit->buffer->size)?
+    (unsigned long)s_end:
+    hedit->buffer->size;
   if (begin == 0 && end == hedit->buffer->size - 1) whole_f = 1;
   while (*file == ' ' || *file == '\t') ++file;
   if (*file == '!') {
@@ -242,7 +246,8 @@ exh_write(struct he_s *hedit, char *file, long begin, long end, int *error)
     while (*file == ' ' || *file == '\t') ++file;
   }
   skip = exh_skipcmd(file, p = (char *)alloca(strlen(file) + 1));
-  util_trunc(file = p);
+  util_trunc(p);
+  file = p;
   if (!*file) {
     if (!(file = current_buffer->path)) {
       he_message(1, "@Abno filename@~");
@@ -261,21 +266,27 @@ exh_write(struct he_s *hedit, char *file, long begin, long end, int *error)
   }
   fclose(fp);
   l = b_copy_to_file(hedit->buffer, file, begin, end - begin + 1);
-  he_message(0, "wrote `%s' - 0x%lx (%li) bytes", file, l, l);
-  if (whole_f) hedit->buffer->modified = 0;
-  if (error) *error = 0;
+  if (l > 0) {
+    he_message(0, "wrote `%s' - 0x%lx (%li) bytes", file, l, l);
+    if (whole_f) hedit->buffer->modified = 0;
+    if (error) *error = 0;
+  } else {
+    he_message(1, "could not write `%s'", file);
+    if (error) *error = 1;
+  }
   return skip;
 }
 /* exh_write */
 
 
-  static char *
-exhcmd_substitute(struct he_s *hedit, char *pattern, long begin, long end)
+  static const char *
+exhcmd_substitute(struct he_s *hedit, const char *pattern, long begin, long end)
 {
-  char *p = pattern;
+  const char *p = pattern;
   char separator = *pattern;
-  char *exp, *replace = 0, *options = 0;
-  char *skip = 0;
+  const char *exp;
+  const char *replace = 0, *options = 0;
+  const char *skip = 0;
   int confirm_f = 0;
   long position, last_match = hedit->position;
   char *replace_str;
@@ -300,11 +311,15 @@ exhcmd_substitute(struct he_s *hedit, char *pattern, long begin, long end)
     if (*exp) {
       p = exh_skip_expression(exp, separator);
       if (p) {
-        *p = 0;
+	char *expbuf = alloca(p - exp + 1);
+	snprintf(expbuf, p - exp + 1, "%s", exp);
+	exp = expbuf;
         replace = p + 1;
         p = exh_skip_replace(replace, separator);
         if (p) {
-          *p = 0;
+	  char *replbuf = alloca(p - replace + 1);
+	  snprintf(replbuf, p - replace + 1, "%s", replace);
+	  replace = replbuf;
           options = p + 1;
           while (*p && *p != ';') ++p;
           if (*p) skip = p + 1; else skip = 0;
@@ -367,11 +382,11 @@ exhcmd_substitute(struct he_s *hedit, char *pattern, long begin, long end)
 	if (!confirm_f)
 	  goto substitute;
 	else {
-	  /* We want to show the user the match found so we can query him/her
+	  /* We want to show the user the match found so we can ask him/her
 	   * if the match should be replaced.  We'll do this by selecting the
            * match.
            * NOTE: `match_len' and/or `replace_len' might be zero.  If both
-           *   are zero, we won't query the user.
+           *   are zero, we won't ask the user.
 	   */
 	  int choice;
           if (!match_len && !replace_len) goto substitute;
@@ -417,8 +432,15 @@ substitute:
         end -= match_len - replace_len;
 	/* create an entry for the undo-list */
         if (match_len) {
-          data = (char *)malloc(match_len);
-          b_read(hedit->buffer, data, position, match_len);
+          data = (char *)malloc_fatal(match_len);
+	  if (b_read(hedit->buffer, data, position, match_len) < 0) {
+	    he_message(1, "could not read data from the internal buffer");
+	    free((char *)data);
+	    free((char *)replace_str);
+	    he_cancel_selection(hedit);
+	    he_update_screen(hedit);
+	    goto exit_exh_substitute;
+	  }
           he_subcommand(hedit, 0, position, match_len, data);
         }
         if (replace_len)
@@ -430,9 +452,22 @@ substitute:
           /* if nothing gets replaced by nothing, we won't count that
            * as a substitution, so there won't be an entry in the undo-
            * list if nothing got changed. */
-	if (match_len) b_delete(hedit->buffer, position, match_len);
+	if (match_len)
+	  if (b_delete(hedit->buffer, position, match_len) < 0) {
+	    he_message(1, "could not remove data from the internal buffer");
+	    free((char *)replace_str);
+	    he_cancel_selection(hedit);
+	    he_update_screen(hedit);
+	    goto exit_exh_substitute;
+	  }
 	if (replace_len) {
-          b_insert(hedit->buffer, position, replace_len);
+          if (b_insert(hedit->buffer, position, replace_len) < 0) {
+	    he_message(1, "could not insert data into the internal buffer");
+	    free((char *)replace_str);
+	    he_cancel_selection(hedit);
+	    he_update_screen(hedit);
+	    goto exit_exh_substitute;
+	  }
           b_write(hedit->buffer, replace_str, position, replace_len);
         }
 	if (confirm_f && match_len) {
@@ -454,20 +489,20 @@ exit_exh_substitute:
 }
 /* exhcmd_substitute */
 
-  static char *
-exhcmd_write(struct he_s *hedit, char *file, long begin, long end)
+  static const char *
+exhcmd_write(struct he_s *hedit, const char *file, long begin, long end)
 {
   return exh_write(hedit, file, begin, end, 0);
 }
 /* exhcmd_write */
 
-  static char *
-exhcmd_skip(struct he_s *hedit, char *args)
+  static const char *
+exhcmd_skip(struct he_s *hedit __unused, const char *args, long begin __unused, long end __unused)
   /* ARGSUSED */
 {
   struct buffer_s *i;
   char *ab = alternate_buffer;
-  char *skip;
+  const char *skip;
 
   skip = exh_skipcmd(args, 0);
   for (i = buffer_list; i; i = i->next)
@@ -488,13 +523,13 @@ exhcmd_skip(struct he_s *hedit, char *args)
 }
 /* exhcmd_skip */
 
-  static char *
-exhcmd_next(struct he_s *hedit, char *args)
+  static const char *
+exhcmd_next(struct he_s *hedit __unused, const char *args, long begin __unused, long end __unused)
   /* ARGSUSED */
 {
   struct buffer_s *i;
   char *ab = alternate_buffer;
-  char *skip;
+  const char *skip;
 
   skip = exh_skipcmd(args, 0);
   if ((i = current_buffer->next)) {
@@ -513,13 +548,13 @@ exhcmd_next(struct he_s *hedit, char *args)
 }
 /* exhcmd_next */
 
-  static char *
-exhcmd_previous(struct he_s *hedit, char *args)
+  static const char *
+exhcmd_previous(struct he_s *hedit __unused, const char *args, long begin __unused, long end __unused)
   /* ARGSUSED */
 {
   struct buffer_s *i;
   char *ab = alternate_buffer;
-  char *skip;
+  const char *skip;
 
   skip = exh_skipcmd(args, 0);
   for (i = buffer_list; i; i = i->next) if (i->next == current_buffer) break;
@@ -539,12 +574,12 @@ exhcmd_previous(struct he_s *hedit, char *args)
 }
 /* exhcmd_previous */
 
-  static char *
-exhcmd_rewind(struct he_s *hedit, char *args)
+  static const char *
+exhcmd_rewind(struct he_s *hedit __unused, const char *args, long begin __unused, long end __unused)
   /* ARGSUSED */
 {
   char *ab = alternate_buffer;
-  char *skip;
+  const char *skip;
 
   skip = exh_skipcmd(args, 0);
   alternate_buffer = current_buffer->hedit->buffer_name;
@@ -560,28 +595,30 @@ exhcmd_rewind(struct he_s *hedit, char *args)
 }
 /* exhcmd_rewind */
 
-  static char *
-exhcmd_wn(struct he_s *hedit, char *file, long begin, long end)
+  static const char *
+exhcmd_wn(struct he_s *hedit, const char *file, long begin, long end)
 {
-  char *skip;
+  const char *skip;
   int error = 0;
 
   skip = exh_write(hedit, file, begin, end, &error);
-  if (!error) exhcmd_next(hedit, "");
+  if (!error) exhcmd_next(hedit, "", 0, 0);
   return skip;
 }
 /* exhcmd_wn */
 
-  static char *
-exhcmd_read(struct he_s *hedit, char *file, long position)
+  static const char *
+exhcmd_read(struct he_s *hedit, const char *file, long position, long end __unused)
 {
   FILE *fp;
   long l;
   char *data;
-  char *skip, *p;
+  const char *skip;
+  char *p;
 
   skip = exh_skipcmd(file, p = (char *)alloca(strlen(file) + 1));
-  util_trunc(file = p);
+  util_trunc(p);
+  file = p;
   while (*file == ' ' || *file == '\t') ++file;
   if (!(fp = fopen(file, "r"))) {
     he_message(0, "@Abcan't open@~ `%s'", file);
@@ -591,9 +628,9 @@ exhcmd_read(struct he_s *hedit, char *file, long position)
   l = b_paste_from_file(hedit->buffer, file, position);
   if (l == -1)
     return skip;
-  data = (char *)malloc(l);
+  data = (char *)malloc_fatal(l);
   fp = fopen(file, "r");
-  if (fread(data, 1, l, fp) != l) {
+  if (fread(data, 1, l, fp) != (unsigned long)l) {
     free(data);
     fclose(fp);
     return skip;
@@ -607,21 +644,24 @@ exhcmd_read(struct he_s *hedit, char *file, long position)
 }
 /* exhcmd_read */
 
-  static char *
-exhcmd_edit(struct he_s *hedit, char *name)
+  static const char *
+exhcmd_edit(struct he_s *hedit __unused, const char *name, long begin __unused, long end __unused)
   /* ARGSUSED */
 {
   char *ab = alternate_buffer;
   struct passwd *pe;
   int new_buffer_f = 0;
   FILE *fp;
-  char *skip, *p;
-  char *path = 0;
+  const char *skip;
+  char *p;
+  const char *path = 0;
   char *user = 0;
-  int uid = getuid();
+  uid_t uid = getuid();
+  char *pathbuf = 0;
 
   skip = exh_skipcmd(name, p = (char *)alloca(strlen(name) + 1));
-  util_trunc(name = p);
+  util_trunc(p);
+  name = p;
   while (*name == ' ' || *name == '\t') ++name;
   if (!strcmp(name, "#")) { /* switch to alternate buffer. */
     if (he_alternate_buffer() < 0) 
@@ -636,7 +676,7 @@ exhcmd_edit(struct he_s *hedit, char *name)
   if (he_select_buffer(name) < 0) {
     new_buffer_f = 1;
     if (*name == '~') {
-      user = strdup(name + 1);
+      user = strdup_fatal(name + 1);
       for (p = user; *p && *p != '/'; ++p);
       *p = 0;
       if (*user) {
@@ -646,9 +686,10 @@ exhcmd_edit(struct he_s *hedit, char *name)
         for (setpwent(); (pe = getpwent());)
           if (uid == pe->pw_uid) break;
       if (pe) {
-        path = (char *)malloc(strlen(name) + strlen(pe->pw_dir) + 4);
-        strcpy(path, pe->pw_dir);
-        strcat(path, name + strlen(user) + 1);
+        pathbuf = (char *)malloc_fatal(strlen(name) + strlen(pe->pw_dir) + 4);
+        strcpy(pathbuf, pe->pw_dir);
+        strcat(pathbuf, name + strlen(user) + 1);
+	path = pathbuf;
       } else
         path = name;
       free(user);
@@ -657,7 +698,7 @@ exhcmd_edit(struct he_s *hedit, char *name)
     if (!(fp = fopen(path, "r"))) {
       he_message(0, "@Abcan't open@~ `%s'", name);
       alternate_buffer = ab;
-      if (path && path != name) free(path);
+      if (pathbuf != NULL) free(pathbuf);
       return skip;
     } else {
       fclose(fp);
@@ -667,31 +708,33 @@ exhcmd_edit(struct he_s *hedit, char *name)
   }
   he_status_message(!new_buffer_f);
   he_refresh_part(current_buffer->hedit, 0, -1);
-  if (path && path != name) free(path);
+  if (pathbuf != NULL) free(pathbuf);
   return skip;
 }
 /* exhcmd_edit */
 
-  static char *
-exhcmd_buffer(struct he_s *hedit, char *name)
+  static const char *
+exhcmd_buffer(struct he_s *hedit, const char *name, long begin __unused, long end __unused)
 {
   char *ab = alternate_buffer;
   long j, k;
   struct buffer_s *i;
   char **list;
   char *c;
-  char *skip, *p;
+  const char *skip;
+  char *p;
 
   skip = exh_skipcmd(name, p = (char *)alloca(strlen(name) + 1));
-  util_trunc(name = p);
+  util_trunc(p);
+  name = p;
   if (!*name) { /* list buffers */
-    int name_maxlen = 0;
+    size_t name_maxlen = 0;
     for (i = buffer_list, k = 0; i; i = i->next, ++k)
       if (strlen(i->hedit->buffer_name) > name_maxlen)
         name_maxlen = strlen(i->hedit->buffer_name);
-    list = (char **)malloc((k + 1) * sizeof(char *));
+    list = (char **)malloc_fatal((k + 1) * sizeof(char *));
     for (i = buffer_list, k = 0; i; i = i->next, ++k) {
-      list[k] = (char *)malloc(strlen(i->hedit->buffer_name) + 512);
+      list[k] = (char *)malloc_fatal(strlen(i->hedit->buffer_name) + 512);
       sprintf(list[k], " `%s' ", i->hedit->buffer_name);
       c = list[k] + strlen(list[k]);
       for (j = name_maxlen - strlen(i->hedit->buffer_name); j >= 0; --j)
@@ -747,17 +790,17 @@ exhcmd_buffer(struct he_s *hedit, char *name)
 }
 /* exhcmd_buffer */
 
-  static char *
-exhcmd_wall(struct he_s *hedit, char *args)
+  static const char *
+exhcmd_wall(struct he_s *hedit, const char *args, long begin __unused, long end __unused)
   /* ARGSUSED */
 {
   struct buffer_s *i;
-  char *skip;
+  const char *skip;
 
   skip = exh_skipcmd(args, 0);
   for (i = buffer_list; i; i = i->next) {
     if (i->loaded_f ? i->hedit->buffer->modified : 0) {
-      exh_save_buffer(hedit, i->hedit->buffer_name, 0, -1, 0);
+      exh_save_buffer(hedit, i->hedit->buffer_name);
 #if 0
       l = b_write_buffer_to_file(i->hedit->buffer, i->path);
       if (l < 0)
@@ -771,12 +814,13 @@ exhcmd_wall(struct he_s *hedit, char *args)
 }
 /* exhcmd_wall */
 
-  static char *
-exhcmd_quit(struct he_s *hedit, char *args)
+  static const char *
+exhcmd_quit(struct he_s *hedit __unused, const char *args, long begin __unused, long end __unused)
   /* ARGSUSED */
 {
   struct buffer_s *i, *j;
-  char *skip, *p;
+  const char *skip;
+  char *p;
   int n;
   static int warn_more_files = 1;
 
@@ -808,16 +852,17 @@ close_all_buffers:
 }
 /* exhcmd_quit */
 
-  static char *
-exhcmd_close(struct he_s *hedit, char *args)
+  static const char *
+exhcmd_close(struct he_s *hedit __unused, const char *aargs, long begin __unused, long end __unused)
   /* ARGSUSED */
 {
   int force_f = 0;
   int nclosed = 0;
   int i;
-  char *skip, *p;
+  const char *skip;
+  char *p, *args;
 
-  skip = exh_skipcmd(args, p = (char *)alloca(strlen(args) + 1));
+  skip = exh_skipcmd(aargs, p = (char *)alloca(strlen(aargs) + 1));
   args = p;
   while (*args == ' ' || *args == '\t') ++args;
   if (*args == '!') ++args, ++force_f;
@@ -851,58 +896,59 @@ exhcmd_close(struct he_s *hedit, char *args)
 }
 /* exhcmd_close */
 
-  static char *
-exhcmd_map(struct he_s *hedit, char *args)
+  static const char *
+exhcmd_map(struct he_s *hedit, const char *args, long begin __unused, long end __unused)
 {
   return exh_map(hedit, args, MAP_COMMAND);
 }
 /* exhcmd_map */
 
-  static char *
-exhcmd_imap(struct he_s *hedit, char *args)
+  static const char *
+exhcmd_imap(struct he_s *hedit, const char *args, long begin __unused, long end __unused)
 {
   return exh_map(hedit, args, MAP_INSERT);
 }
 /* exhcmd_imap */
 
-  static char *
-exhcmd_vmap(struct he_s *hedit, char *args)
+  static const char *
+exhcmd_vmap(struct he_s *hedit, const char *args, long begin __unused, long end __unused)
 {
   return exh_map(hedit, args, MAP_VISUAL);
 }
 /* exhcmd_vmap */
 
-  static char *
-exhcmd_unmap(struct he_s *hedit, char *args)
+  static const char *
+exhcmd_unmap(struct he_s *hedit, const char *args, long begin __unused, long end __unused)
 {
   return exh_unmap(hedit, args, MAP_COMMAND);
 }
 /* exhcmd_unmap */
 
-  static char *
-exhcmd_iunmap(struct he_s *hedit, char *args)
+  static const char *
+exhcmd_iunmap(struct he_s *hedit, const char *args, long begin __unused, long end __unused)
 {
   return exh_unmap(hedit, args, MAP_INSERT);
 }
 /* exhcmd_iunmap */
 
-  static char *
-exhcmd_vunmap(struct he_s *hedit, char *args)
+  static const char *
+exhcmd_vunmap(struct he_s *hedit, const char *args, long begin __unused, long end __unused)
 {
   return exh_unmap(hedit, args, MAP_VISUAL);
 }
 /* exhcmd_vunmap */
 
-  static char *
-exhcmd_set(struct he_s *hedit, char *args)
+  static const char *
+exhcmd_set(struct he_s *hedit, const char *args, long begin __unused, long end __unused)
 {
   char *p, *q, *option;
   enum s_option_e type;
   int no_f;
-  char *skip;
+  const char *skip;
 
   skip = exh_skipcmd(args, p = (char *)alloca(strlen(args) + 1));
-  util_trunc(args = p);
+  util_trunc(p);
+  args = p;
   if (!*args) {
     char **list = s_option_value_list(), **i = list;
     if (*list) {
@@ -914,7 +960,6 @@ exhcmd_set(struct he_s *hedit, char *args)
     free((char *)list);
     return skip;
   }
-  p = args;
   for (; *p;) {
     no_f = 0;
     option = q = p;
@@ -950,7 +995,7 @@ exhcmd_set(struct he_s *hedit, char *args)
       if (type == S_BOOL)
         s_set_option(option, no_f ? "false" : "true", 0);
       else {
-        char *value = s_get_option(option);
+        const char *value = s_get_option(option);
         if (!*value) value = "<unset>";
         he_message(0, "%s=%s@~", option, value);
       }
@@ -960,18 +1005,20 @@ exhcmd_set(struct he_s *hedit, char *args)
 }
 /* exhcmd_set */
 
-  static char *
-exhcmd_delete(struct he_s *hedit, char *args, long begin, long end)
+  static const char *
+exhcmd_delete(struct he_s *hedit, const char *args, long begin, long end)
   /* ARGSUSED */
 {
   long count = end - begin;
   char *data;
-  char *skip, *p;
+  const char *skip;
+  char *p;
 
   skip = exh_skipcmd(args, p = (char *)alloca(strlen(args) + 1));
-  util_trunc(args = p);
+  util_trunc(p);
+  args = p;
   if (!count) return skip;
-  data = (char *)malloc(count);
+  data = (char *)malloc_fatal(count);
   b_read(hedit->buffer, data, begin, count);
   if (!kill_buffer)
     kill_buffer = new_buffer(0);
@@ -991,15 +1038,17 @@ exhcmd_delete(struct he_s *hedit, char *args, long begin, long end)
 }
 /* exhcmd_delete */
 
-  static char *
-exhcmd_yank(struct he_s *hedit, char *args, long begin, long end)
+  static const char *
+exhcmd_yank(struct he_s *hedit, const char *args, long begin, long end)
   /* ARGSUSED */
 {
   long count = end - begin;
-  char *skip, *p;
+  const char *skip;
+  char *p;
 
   skip = exh_skipcmd(args, p = (char *)alloca(strlen(args) + 1));
-  util_trunc(args = p);
+  util_trunc(p);
+  args = p;
   if (kill_buffer) b_clear(kill_buffer); else kill_buffer = new_buffer(0);
   if (!count) return skip;
   b_insert(kill_buffer, 0, count);
@@ -1010,21 +1059,21 @@ exhcmd_yank(struct he_s *hedit, char *args, long begin, long end)
 }
 /* exhcmd_yank */
 
-  static char *
-exhcmd_version(struct he_s *hedit, char *args)
+  static const char *
+exhcmd_version(struct he_s *hedit __unused, const char *args, long begin __unused, long end __unused)
   /* ARGSUSED */
 {
-  char *skip = exh_skipcmd(args, 0);
+  const char *skip = exh_skipcmd(args, 0);
   hexer_version();
   return skip;
 }
 /* exhcmd_version */
 
-  static char *
-exhcmd_zz(struct he_s *hedit, char *args)
+  static const char *
+exhcmd_zz(struct he_s *hedit, const char *args, long begin __unused, long end __unused)
 {
   int k;
-  char *skip;
+  const char *skip;
 
   skip = exh_skipcmd(args, 0);
   if ((k = HE_LINE(hedit->position - hedit->screen_offset)) < hx_lines / 2 - 1)
@@ -1036,10 +1085,10 @@ exhcmd_zz(struct he_s *hedit, char *args)
 }
 /* exhcmd_zz */
 
-  static char *
-exhcmd_zt(struct he_s *hedit, char *args)
+  static const char *
+exhcmd_zt(struct he_s *hedit, const char *args, long begin __unused, long end __unused)
 {
-  char *skip;
+  const char *skip;
 
   skip = exh_skipcmd(args, 0);
   he_scroll_up(hedit, HE_LINE(hedit->position - hedit->screen_offset));
@@ -1048,10 +1097,10 @@ exhcmd_zt(struct he_s *hedit, char *args)
 }
 /* exhcmd_zt */
 
-  static char *
-exhcmd_zb(struct he_s *hedit, char *args)
+  static const char *
+exhcmd_zb(struct he_s *hedit, const char *args, long begin __unused, long end __unused)
 {
-  char *skip;
+  const char *skip;
 
   skip = exh_skipcmd(args, 0);
   he_scroll_down(hedit,
@@ -1061,13 +1110,13 @@ exhcmd_zb(struct he_s *hedit, char *args)
 }
 /* exhcmd_zb */
 
-  static char *
-exhcmd_help(struct he_s *hedit, char *args)
+  static const char *
+exhcmd_help(struct he_s *hedit, const char *args, long begin __unused, long end __unused)
 {
   int pid1, pid2, status, x = 0;
   int pipefd[2];
   char *vp[64], *p;
-  char *skip;
+  const char *skip;
   int i;
 
   skip = exh_skipcmd(args, 0);
@@ -1106,12 +1155,8 @@ exhcmd_help(struct he_s *hedit, char *args)
     close(pipefd[1]);
     goto exit_exhcmd_help;
   } else if (!pid2) { /* child */
-    size_t len, n;
-    ssize_t res;
-
-    for (n = 0, len = strlen(helptext); n < len; n += res)
-      if ((res = write(pipefd[1], helptext + n, len - n)) == -1)
-	exit(1);
+    if (write_buf(pipefd[1], helptext, strlen(helptext)) < 0)
+      exit(1);
     close(pipefd[1]);
     exit(0);
   }
@@ -1133,16 +1178,18 @@ exit_exhcmd_help:
 }
 /* exhcmd_help */
 
-  static char *
-exhcmd_exit(struct he_s *hedit, char *args)
+  static const char *
+exhcmd_exit(struct he_s *hedit, const char *args, long begin __unused, long end __unused)
 {
   struct buffer_s *i;
   int cant_write_f = 0;
   long k;
-  char *skip, *p;
+  const char *skip;
+  char *p;
 
   skip = exh_skipcmd(args, p = (char *)alloca(strlen(args) + 1));
-  util_trunc(args = p);
+  util_trunc(p);
+  args = p;
   for (i = buffer_list; i; i = i->next)
     if (i->loaded_f && i->path ? i->hedit->buffer->modified : 0) {
       if (i->hedit->read_only) {
@@ -1161,41 +1208,41 @@ exhcmd_exit(struct he_s *hedit, char *args)
         }
       }
     }
-  if (!cant_write_f) exhcmd_quit(hedit, "");
+  if (!cant_write_f) exhcmd_quit(hedit, "", 0, 0);
   return skip;
 }
 /* exhcmd_exit */
 
 const struct exh_cmd_s exh_commands[] = {
-  { "substitute", 's', (exh_fn)exhcmd_substitute, 1, 0 },
-  { "write", 'w', (exh_fn)exhcmd_write, 1, 1 },
-  { "read", 'r', (exh_fn)exhcmd_read, 0, 1 },
-  { "edit", 'e', (exh_fn)exhcmd_edit, 1, 3 },
-  { "buffer", 'b', (exh_fn)exhcmd_buffer, 1, 2 },
-  { "next", 'n', (exh_fn)exhcmd_next, 0, 0 },
-  { "previous", 'N', (exh_fn)exhcmd_previous, 0, 0 },
-  { "rewind", 0, (exh_fn)exhcmd_rewind, 0, 0 },
-  { "skip", 'S', (exh_fn)exhcmd_skip, 0, 0 },
-  { "wn", 0, (exh_fn)exhcmd_wn, 1, 1 },
-  { "wall", 0, (exh_fn)exhcmd_wall, 0, 0 },
-  { "quit", 'q', (exh_fn)exhcmd_quit, 0, 0 },
-  { "close", 'c', (exh_fn)exhcmd_close, 0, 2 },
-  { "map", 0, (exh_fn)exhcmd_map, 0, 0 },
-  { "imap", 0, (exh_fn)exhcmd_imap, 0, 0 },
-  { "vmap", 0, (exh_fn)exhcmd_vmap, 0, 0 },
-  { "unmap", 0, (exh_fn)exhcmd_unmap, 0, 0 },
-  { "iunmap", 0, (exh_fn)exhcmd_iunmap, 0, 0 },
-  { "vunmap", 0, (exh_fn)exhcmd_vunmap, 0, 0 },
-  { "set", 0, (exh_fn)exhcmd_set, 0, 4 },
-  { "delete", 'd', (exh_fn)exhcmd_delete, 0, 0 },
-  { "yank", 'y', (exh_fn)exhcmd_yank, 0, 0 },
-  { "version", 0, (exh_fn)exhcmd_version, 0, 0 },
-  { "zz", 0, (exh_fn)exhcmd_zz, 0, 0 },
-  { "zt", 0, (exh_fn)exhcmd_zt, 0, 0 },
-  { "zb", 0, (exh_fn)exhcmd_zb, 0, 0 },
-  { "wq", 0, (exh_fn)exhcmd_exit, 0, 0 },
-  { "help", 'h', (exh_fn)exhcmd_help, 0, 0 },
-  { "exit", 'x', (exh_fn)exhcmd_exit, 0, 0 },
+  { "substitute", 's', exhcmd_substitute, 1, 0 },
+  { "write", 'w', exhcmd_write, 1, 1 },
+  { "read", 'r', exhcmd_read, 0, 1 },
+  { "edit", 'e', exhcmd_edit, 1, 3 },
+  { "buffer", 'b', exhcmd_buffer, 1, 2 },
+  { "next", 'n', exhcmd_next, 0, 0 },
+  { "previous", 'N', exhcmd_previous, 0, 0 },
+  { "rewind", 0, exhcmd_rewind, 0, 0 },
+  { "skip", 'S', exhcmd_skip, 0, 0 },
+  { "wn", 0, exhcmd_wn, 1, 1 },
+  { "wall", 0, exhcmd_wall, 0, 0 },
+  { "quit", 'q', exhcmd_quit, 0, 0 },
+  { "close", 'c', exhcmd_close, 0, 2 },
+  { "map", 0, exhcmd_map, 0, 0 },
+  { "imap", 0, exhcmd_imap, 0, 0 },
+  { "vmap", 0, exhcmd_vmap, 0, 0 },
+  { "unmap", 0, exhcmd_unmap, 0, 0 },
+  { "iunmap", 0, exhcmd_iunmap, 0, 0 },
+  { "vunmap", 0, exhcmd_vunmap, 0, 0 },
+  { "set", 0, exhcmd_set, 0, 4 },
+  { "delete", 'd', exhcmd_delete, 0, 0 },
+  { "yank", 'y', exhcmd_yank, 0, 0 },
+  { "version", 0, exhcmd_version, 0, 0 },
+  { "zz", 0, exhcmd_zz, 0, 0 },
+  { "zt", 0, exhcmd_zt, 0, 0 },
+  { "zb", 0, exhcmd_zb, 0, 0 },
+  { "wq", 0, exhcmd_exit, 0, 0 },
+  { "help", 'h', exhcmd_help, 0, 0 },
+  { "exit", 'x', exhcmd_exit, 0, 0 },
   { 0, 0, (exh_fn)0, 0, 0 }
 };
 
