@@ -2,7 +2,7 @@
  */
 
 /* Copyright (c) 1995,1996 Sascha Demetrio
- * Copyright (c) 2009, 2014 Peter Pentchev
+ * Copyright (c) 2009, 2014, 2015 Peter Pentchev
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -10,14 +10,14 @@
  * are met:
  * 1. Redistributions of source code must retain the above copyright
  *    notice, this list of conditions and the following disclaimer.
- *    If you modify any part of HEXER and resitribute it, you must add
+ *    If you modify any part of HEXER and redistribute it, you must add
  *    a notice to the `README' file and the modified source files containing
  *    information about the  changes you made.  I do not want to take
  *    credit or be blamed for your modifications.
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- *    If you modify any part of HEXER and resitribute it in binary form,
+ *    If you modify any part of HEXER and redistribute it in binary form,
  *    you must supply a `README' file containing information about the
  *    changes you made.
  * 3. The name of the developer may not be used to endorse or promote
@@ -45,6 +45,7 @@
 
 #include "config.h"
 
+#include <alloca.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -60,6 +61,7 @@
 #include "edit.h"
 #include "readline.h"
 #include "tio.h"
+#include "util.h"
 
 int he_hex_column[] = { 12, 15, 18, 21, 24, 27, 30, 33,
                         37, 40, 43, 46, 49, 52, 55, 58 };
@@ -84,7 +86,7 @@ const struct he_refresh_s NO_REFRESH =
 int he_message_wait = 10;
 
   static char *
-he_bigendian(long x)
+he_bigendian(unsigned long x)
 {
   static char bigendian[4];
 
@@ -96,15 +98,16 @@ he_bigendian(long x)
 }
 /* he_bigendian */
 
-  static long
-he_bigendian_to_long(char *bigendian)
+  static unsigned long
+he_bigendian_to_ulong(char *bigendian)
 {
-  long x;
+  const unsigned char * const be = (const unsigned char *)bigendian;
+  unsigned long x;
 
-  x = (unsigned char)bigendian[3]
-      + ((unsigned char)bigendian[2] << 8)
-      + ((unsigned char)bigendian[1] << 16)
-      + ((unsigned char)bigendian[0] << 24);
+  x = (unsigned long)be[3]
+      + ((unsigned long)be[2] << 8)
+      + ((unsigned long)be[1] << 16)
+      + ((unsigned long)be[0] << 24);
   return x;
 }
 /* he_bigendian_to_long */
@@ -120,23 +123,88 @@ he_digit(int key)
 /* he_digit */
 
 
-  void
-he_refresh_lines(hedit, first, last)
-  struct he_s *hedit;
-  int first, last;
+static int refresh_mergeable(const int last1, const int first2, const int last2, int * const nlast)
 {
-  if (!hedit->refresh.flag) ++hedit->refresh.flag;
-  hedit->refresh.first[hedit->refresh.parts] = first;
-  hedit->refresh.last[hedit->refresh.parts] = last;
-  ++hedit->refresh.parts;
-  assert(hedit->refresh.parts <= HE_REFRESH_MAX_PARTS);
+  if (last1 == -1) {
+    *nlast = -1;
+    return 1;
+  } else if (first2 == 0 || last1 >= first2 - 1) {
+    *nlast = last2 == -1? -1:
+      last2 > last1? last2: last1;
+    return 1;
+  }
+  return 0;
+}
+
+  void
+he_refresh_lines(struct he_s * const hedit, const int first, const int last)
+{
+  struct he_refresh_s * const refresh = &hedit->refresh;
+  if (!refresh->flag) refresh->flag = 1;
+
+  /* Find out which part this should be inserted before. */
+  const size_t parts = refresh->parts;
+  size_t next;
+  for (next = 0; next < parts; next++)
+    if (refresh->first[next] >= first)
+      break;
+  int nlast;
+  size_t merge_first;
+  if (next > 0 &&
+    refresh_mergeable(refresh->last[next - 1], first, last, &nlast)) {
+    /* It overlaps with the previous one. */
+    refresh->last[next - 1] = nlast;
+    merge_first = next - 1;
+  } else if (next < parts &&
+    refresh_mergeable(last, refresh->first[next], refresh->last[next], &nlast)) {
+    /* It overlaps with the next one. */
+    if (refresh->first[next] > first)
+      refresh->first[next] = first;
+    refresh->last[next] = nlast;
+    merge_first = next;
+  } else {
+    /* Make room for one more! */
+    assert(parts < HE_REFRESH_MAX_PARTS);
+    memmove(refresh->first + next + 1, refresh->first + next,
+      (parts - next) * sizeof(refresh->first[0]));
+    memmove(refresh->last + next + 1, refresh->last + next,
+      (parts - next) * sizeof(refresh->last[0]));
+    refresh->first[next] = first;
+    refresh->last[next] = last;
+    refresh->parts++;
+    return;
+  }
+
+  /**
+   * Right, we extended some refresh entry, let's see how much further
+   * we can go.
+   */
+  for (next = merge_first + 1; next < parts; next++) {
+    if (!refresh_mergeable(refresh->last[merge_first],
+      refresh->first[next], refresh->last[next], &nlast))
+      break;
+    refresh->last[merge_first] = nlast;
+  }
+  /* If we merged at least one, move the rest back.
+   * Right now 'next' points to the first one that could not be merged at
+   * the end of 'merge_first'.
+   */
+  const size_t removed = next - (merge_first + 1);
+  if (removed > 0) {
+    const size_t left = parts - next;
+    if (left > 0) {
+      memmove(refresh->first + merge_first + 1, refresh->first + next,
+	left * sizeof(refresh->first[0]));
+      memmove(refresh->last + merge_first + 1, refresh->last + next,
+	left * sizeof(refresh->last[0]));
+    }
+    refresh->parts -= removed;
+  }
 }
 /* he_refresh_lines */
 
   void 
-he_refresh_part(hedit, pos1, pos2)
-  struct he_s *hedit;
-  long pos1, pos2;
+he_refresh_part(struct he_s * const hedit, long pos1, long pos2)
 {
   long i, j, k;
   int update_f = 0;
@@ -166,16 +234,14 @@ he_refresh_part(hedit, pos1, pos2)
 /* he_refresh_part */
 
   int
-he_refresh_check(hedit)
-  struct he_s *hedit;
+he_refresh_check(struct he_s *hedit)
 {
   return hedit->refresh.flag || he_messages;
 }
 /* he_refresh_check */
 
   void
-he_refresh_screen(hedit)
-  const struct he_s *hedit;
+he_refresh_screen(const struct he_s *hedit)
 {
   he_refresh_all(current_buffer->hedit);
   he_update_screen(current_buffer->hedit);
@@ -190,8 +256,7 @@ he_refresh_screen(hedit)
 
 
   void
-he_free_command(command)
-  struct he_command_s *command;
+he_free_command(struct he_command_s * const command)
   /* Free the memory allocated by `command' and all the following commands
    * in the list.
    */
@@ -211,9 +276,7 @@ he_free_command(command)
 /* he_free_command */
 
   void
-he_compound_comand(hedit, command)
-  struct he_s *hedit;
-  struct he_command_s *command;
+he_compound_comand(struct he_s *hedit, struct he_command_s *command)
 {
   char *buf;
   size_t sz, pos;
@@ -227,6 +290,12 @@ he_compound_comand(hedit, command)
     long swap_position = ftell(hedit->undo.swapfile);
     int n;
 
+    if (swap_position == -1) {
+      /* TODO: some kind of error indication */
+      fclose(hedit->undo.swapfile);
+      hedit->swapping = 0;
+      return;
+    }
     /* Write the command to the swapfile.
      */
     sz = 4 + 1 + 4 + 4;
@@ -280,12 +349,7 @@ he_compound_comand(hedit, command)
 /* he_compound_comand */
 
   void
-he_subcommand(hedit, type, position, count, data)
-  struct he_s *hedit;
-  int type;
-  long position;
-  long count;
-  char *data;
+he_subcommand(struct he_s *hedit, int type, unsigned long position, unsigned long count, char *data)
   /* Update the undo-list of `hedit' by inserting the given subcommand.
    * The command is *not* performed by calling `he_subcommand()'.
    * A sequence of subcommands must be terminated by calling
@@ -296,10 +360,10 @@ he_subcommand(hedit, type, position, count, data)
 {
   static struct he_command_s *command = 0;
     /* Maybe `command' will become part of `struct he_s' instead of being
-     * remembered statcially.  To stay compatible `hedit' should be 
+     * remembered statically.  To stay compatible `hedit' should be 
      * specified in all calls to `he_subcommand()'.
      */
-  static long last_position;
+  static unsigned long last_position;
   static int again;
   struct he_command_s *i;
 
@@ -309,14 +373,14 @@ he_subcommand(hedit, type, position, count, data)
     command = 0;
   } else { /* (type >= 0) */
     if (!command) {
-      command = (struct he_command_s *)malloc(sizeof(*command));
+      command = (struct he_command_s *)malloc_fatal(sizeof(*command));
       again = 1;
       last_position = position;
       i = command;
     } else {
       if (again) if (position != last_position) again = 0;
       for (i = command; i->next_subcommand; i = i->next_subcommand);
-      i->next_subcommand = (struct he_command_s *)malloc(sizeof(*command));
+      i->next_subcommand = (struct he_command_s *)malloc_fatal(sizeof(*command));
       i = i->next_subcommand;
     }
     i->next_subcommand = 0;
@@ -331,9 +395,7 @@ he_subcommand(hedit, type, position, count, data)
 /* he_subcommand */
 
   long
-he_do_command(hedit, command)
-  struct he_s *hedit;
-  struct he_command_s *command;
+he_do_command(struct he_s *hedit, struct he_command_s *command)
   /* Perform the compound command `command'.  The return value is the
    * position of the last change made.
    */
@@ -395,7 +457,7 @@ he_rewind_command(struct he_s *hedit)
   fseek(hedit->undo.swapfile, -4, SEEK_CUR);
   if (fread(bigendian, 4, 1, hedit->undo.swapfile) < 1)
     return -1;
-  position = he_bigendian_to_long(bigendian);
+  position = he_bigendian_to_ulong(bigendian);
   if (!position) return -1;
   fseek(hedit->undo.swapfile, position, SEEK_SET);
   return position;
@@ -410,20 +472,20 @@ he_read_command(struct he_s *hedit)
    */
 {
   struct he_command_s *command, *c;
-  long n_subcommands;
+  unsigned long n_subcommands;
   char bigendian[4];
-  int i;
+  unsigned i;
 
   if (fread(bigendian, 4, 1, hedit->undo.swapfile) != 1)
     return 0;
-  n_subcommands = he_bigendian_to_long(bigendian);
+  n_subcommands = he_bigendian_to_ulong(bigendian);
   if (!n_subcommands) {
     fseek(hedit->undo.swapfile, -4, SEEK_CUR);
     return 0;
   }
   if (fgetc(hedit->undo.swapfile) == EOF)
     return 0;
-  c = command = (struct he_command_s *)malloc(sizeof(struct he_command_s));
+  c = command = (struct he_command_s *)malloc_fatal(sizeof(struct he_command_s));
   for (i = 1; c; ++i) {
     int n;
     c->next_command = 0;
@@ -434,16 +496,16 @@ he_read_command(struct he_s *hedit)
     c->type = !!n;
     if (fread(bigendian, 4, 1, hedit->undo.swapfile) != 1)
       return 0;
-    c->position = he_bigendian_to_long(bigendian);
+    c->position = he_bigendian_to_ulong(bigendian);
     if (fread(bigendian, 4, 1, hedit->undo.swapfile) != 1)
       return 0;
-    c->count = he_bigendian_to_long(bigendian);
-    c->data = (char *)malloc(c->count);
+    c->count = he_bigendian_to_ulong(bigendian);
+    c->data = (char *)malloc_fatal(c->count);
     if (fread(c->data, 1, c->count, hedit->undo.swapfile) != c->count)
       return 0;
     if (i < n_subcommands) {
       c->next_subcommand =
-        (struct he_command_s *)malloc(sizeof(struct he_command_s));
+        (struct he_command_s *)malloc_fatal(sizeof(struct he_command_s));
       c = c->next_subcommand;
     } else {
       c->next_subcommand = 0;
@@ -536,7 +598,7 @@ fail:
   static void
 he_again(struct he_s *hedit, long position)
   /* Walk backwards through the undo list, until a command is found with
-   * the `again' flag set and reperfom this command on position `position'.
+   * the `again' flag set and reperform this command at position `position'.
    * If no such command is found, a beep is "returned".
    */
 {
@@ -575,30 +637,32 @@ he_again(struct he_s *hedit, long position)
        * positions.  If `command' is an insert command, the field `data'
        * will be copied to `command2'.  If `command' is a delete command,
        * we'll read the `data' field from the buffer.  In this case it may
-       * be nesseccary to adjust `j->count' in `command2'.
+       * be necessary to adjust `j->count' in `command2'.
        */
       struct he_command_s *command2, *j;
-      command2 = (struct he_command_s *)malloc(sizeof(struct he_command_s));
+      command2 = (struct he_command_s *)malloc_fatal(sizeof(struct he_command_s));
       for (i = command, j = command2; i;) {
         *j = *i;
         j->next_command = 0;
         j->prev_command = 0;
         if (i->count) {
-          j->data = (char *)malloc(i->count);
+          j->data = (char *)malloc_fatal(i->count);
           if (i->type) /* insert */
             memcpy(j->data, i->data, i->count);
           else { /* delete */
             long x = b_read(hedit->buffer, j->data, position, j->count);
-            if (x < j->count) {
+	    if (x > 0) {
+	      assert(0); /* FIXME: better handling */
+	    } else if ((unsigned long)x < j->count) {
               /* `x > j->count' is impossible here. */
-              j->data = (char *)realloc(j->data, x);
+              j->data = (char *)realloc_fatal(j->data, x);
               j->count = x;
             }
           }
         }
         if (i->next_subcommand) {
           j->next_subcommand =
-            (struct he_command_s *)malloc(sizeof(struct he_command_s));
+            (struct he_command_s *)malloc_fatal(sizeof(struct he_command_s));
           j = j->next_subcommand;
           i = i->next_subcommand;
         } else
@@ -620,13 +684,13 @@ he_delete(struct he_s *hedit, long count, int dont_save)
   /* If `dont_save == 1' the data deleted is not copied to the `kill_buffer'.
    */
 {
-  long start;
+  unsigned long start;
   char *data;
 
   if (hedit->begin_selection >= 0 &&
       hedit->end_selection >= hedit->begin_selection) {
-    start = hedit->begin_selection;
-    count = hedit->end_selection - start + 1;
+    start = (unsigned long)hedit->begin_selection;
+    count = (unsigned long)hedit->end_selection - start + 1;
     he_cancel_selection(hedit);
     he_update_screen(hedit);
   } else {
@@ -636,7 +700,7 @@ he_delete(struct he_s *hedit, long count, int dont_save)
       count = hedit->buffer->size - start;
   }
   if (!count) return;
-  data = (char *)malloc(count);
+  data = (char *)malloc_fatal(count);
   b_read(hedit->buffer, data, start, count);
   if (!dont_save) {
     if (!kill_buffer)
@@ -649,7 +713,7 @@ he_delete(struct he_s *hedit, long count, int dont_save)
   he_subcommand(hedit, 0, start, count, data);
   he_subcommand(hedit, -1, 0, 0, 0);
   b_delete(hedit->buffer, start, count);
-  if (hedit->position > start) hedit->position -= count - 1;
+  if (hedit->position > (long)start) hedit->position -= count - 1;
   he_refresh_part(hedit, start, hedit->buffer->size + count - 1);
 
   if (count > 1)
@@ -668,7 +732,7 @@ he_paste(struct he_s *hedit, long count)
   if (kill_buffer ? !(c = kill_buffer->size) : 1) return;
   if (count < 1) count = 1;
   while (count--) {
-    data = (char *)malloc(c);
+    data = (char *)malloc_fatal(c);
     b_read(kill_buffer, data, 0, c);
     b_insert(hedit->buffer, hedit->position, c);
     b_write(hedit->buffer, data, hedit->position, c);
@@ -692,8 +756,8 @@ he_paste_over(struct he_s *hedit, long count)
   if (kill_buffer ? !(c = kill_buffer->size) : 1) return;
   if (count < 1) count = 1;
 
-  data_insert = (char *)malloc(c * count);
-  data_delete = (char *)malloc(c * count);
+  data_insert = (char *)malloc_fatal(c * count);
+  data_delete = (char *)malloc_fatal(c * count);
   b_read(kill_buffer, data_insert, 0, c);
   for (i = 1; i < count; ++i) memcpy(data_insert + c * i, data_insert, c);
   bytes_deleted = b_read(hedit->buffer, data_delete,
@@ -701,7 +765,7 @@ he_paste_over(struct he_s *hedit, long count)
   b_write_append(hedit->buffer, data_insert, hedit->position, c * count);
   if (bytes_deleted < c * count)
     data_delete =
-      (char *)realloc(data_delete, bytes_deleted + !bytes_deleted);
+      (char *)realloc_fatal(data_delete, bytes_deleted + !bytes_deleted);
   he_subcommand(hedit, 0, hedit->position, bytes_deleted, data_delete);
   he_subcommand(hedit, 1, hedit->position, c * count, data_insert);
   he_subcommand(hedit, -1, 0, 0, 0);
@@ -865,8 +929,7 @@ he_line(const struct he_s *hedit, long position)
 /* he_line */
 
   static void
-he_set_cursor(hedit)
-  const struct he_s *hedit;
+he_set_cursor(const struct he_s *hedit)
 {
   int l, c, i;
 
@@ -925,14 +988,14 @@ he_motion(struct he_s *hedit, int key, long count)
   case 'l':
   case HXKEY_RIGHT:
     hedit->position += count;
-    if (hedit->position > hedit->buffer->size)
+    if (hedit->position > (long)hedit->buffer->size)
       hedit->position = hedit->buffer->size;
     break;
   case 'f' & 0x1f: /* C-f */
   case HXKEY_PAGE_DOWN:
     i = hedit->position;
     hedit->position += (hx_lines - 2) * 16;
-    if (hedit->position > hedit->buffer->size)
+    if (hedit->position > (long)hedit->buffer->size)
       hedit->position = hedit->buffer->size;
     hedit->screen_offset += ((hedit->position - i) / 16) * 16;
     if (hedit->position != i) he_refresh_all(hedit);
@@ -949,7 +1012,7 @@ he_motion(struct he_s *hedit, int key, long count)
   case 'd' & 0x1f: /* C-d */
     i = hedit->position;
     hedit->position += ((hx_lines - 1) / 2) * 16;
-    if (hedit->position > hedit->buffer->size)
+    if (hedit->position > (long)hedit->buffer->size)
       hedit->position = hedit->buffer->size;
     hedit->screen_offset += ((hedit->position - i) / 16) * 16;
     if (hedit->position != i) he_refresh_all(hedit);
@@ -1019,8 +1082,7 @@ he_end_selection(struct he_s *hedit)
 /* he_end_selection */
 
   void
-he_cancel_selection(hedit)
-  struct he_s *hedit;
+he_cancel_selection(struct he_s *hedit)
   /* Cancel the current selection.
    */
 {
@@ -1037,9 +1099,7 @@ he_cancel_selection(hedit)
 /* he_cancel_selection */
 
   void
-he_select(hedit, begin, end)
-  struct he_s *hedit;
-  long begin, end;
+he_select(struct he_s *hedit, unsigned long begin, unsigned long end)
 {
   if (hedit->begin_selection >= 0) he_cancel_selection(hedit);
   assert(begin <= end);
@@ -1113,7 +1173,7 @@ he_command(struct he_s *hedit, int key, long count)
     if (count < 0)
       hedit->position = hedit->buffer->size;
     else {
-      if (count > hedit->buffer->size) {
+      if (count > (long)hedit->buffer->size) {
         count = hedit->buffer->size;
         he_message(0, "only 0x%lx (%li) bytes", count, count);
       }
@@ -1175,12 +1235,11 @@ he_command(struct he_s *hedit, int key, long count)
    */
 #define HE_CASE_COMMAND _HE_CASE_COMMAND_NO_V: case 'v'
 #define HE_CASE_COMMAND_INSERT case 'l' & 0x1f: case 'g' & 0x1f
-  /* Commands that should be availabe in insert mode.
+  /* Commands that should be available in insert mode.
    */
 
   static void
-he_visual_mode(hedit)
-  struct he_s *hedit;
+he_visual_mode(struct he_s *hedit)
 {
   int key;
   long anchor = hedit->position;
@@ -1194,7 +1253,7 @@ he_visual_mode(hedit)
     he_message(1, "@Abthe buffer is empty@~");
     return;
   }
-  if (hedit->position == hedit->buffer->size) {
+  if (hedit->position == (long)hedit->buffer->size) {
     --hedit->position;
     he_update_screen(hedit);
   }
@@ -1229,7 +1288,7 @@ he_visual_mode(hedit)
     HE_CASE_MOTION:
     HE_CASE_MOTION_HJKL:
       he_motion(hedit, key, count);
-      if (hedit->position == hedit->buffer->size) --hedit->position;
+      if (hedit->position == (long)hedit->buffer->size) --hedit->position;
       he_update_screen(hedit);
       if (hedit->position == last_position) break;
       if (anchor > hedit->position) {
@@ -1299,13 +1358,12 @@ exit_visual_mode2:
   tio_set_cursor(1); /* normal cursor. */
   f = hedit->refresh.message_f;
   he_update_screen(hedit);
-  if (!f) he_message(0, "");
+  if (!f) he_message(0, " ");
 }
 /* he_visual_mode */
 
   static long
-he_get_counter(hedit)
-  struct he_s *hedit;
+he_get_counter(struct he_s *hedit)
   /* Read a counter from the keyboard.  If this is terminated by pressing
    * <esc>, the return value is -1.  If another non-digit key is read,
    * it is put back via `tio_ungetch()'.  Reading the counter can be
@@ -1315,7 +1373,7 @@ he_get_counter(hedit)
   long count = -1;
   int key, digit;
   enum mode_e { OCT = 8, DEC = 10, HEX = 16 } mode = DEC;
-  char *fmt = 0, *prefix = 0;
+  const char *fmt = 0, *prefix = 0;
 
   tio_goto_line(hx_lines - 1);
   tio_return();
@@ -1327,7 +1385,7 @@ he_get_counter(hedit)
       if (window_changed) he_refresh_screen(hedit);
       break;
     HE_CASE_COMMAND_INSERT:
-      /* The commands that are availabe in insert mode do not affect and
+      /* The commands that are available in insert mode do not affect and
        * are not affected by the counter.
        */
       he_command(hedit, key, 0);
@@ -1377,22 +1435,22 @@ he_get_counter(hedit)
       int indent = 60;
       sprintf(arg, fmt, count);
       tio_goto_line(hx_lines - 1);
-      if (indent + strlen(arg) > hx_columns - 1)
+      if (indent + (ssize_t)strlen(arg) > hx_columns - 1)
         indent = hx_columns - 1 - strlen(arg);
       if (indent < 0) indent = 0;
       tio_return();
       tio_right(indent);
-      tio_printf(arg);
+      tio_printf("%s", arg);
       tio_clear_to_eol();
     } else {
       int indent = 60;
       tio_goto_line(hx_lines - 1);
-      if (indent + strlen(prefix) > hx_columns - 1)
+      if (indent + (ssize_t)strlen(prefix) > hx_columns - 1)
         indent = hx_columns - 1 - strlen(prefix);
       if (indent < 0) indent = 0;
       tio_return();
       tio_right(indent);
-      tio_printf(prefix);
+      tio_printf("%s", prefix);
       tio_clear_to_eol();
     }
     tio_flush();
@@ -1440,10 +1498,7 @@ restart:
 /* he_verbatim */
 
   static void
-he_insert_mode(hedit, replace_mode, count)
-  struct he_s *hedit;
-  int replace_mode;
-  long count;
+he_insert_mode(struct he_s *hedit, int replace_mode, long count)
   /* `replace_mode == 0':  Normal insert mode.
    * `replace_mode == 1':  Replace mode.  Characters are appended if the
    *                       end of the buffer is reached.
@@ -1475,11 +1530,11 @@ he_insert_mode(hedit, replace_mode, count)
           /* Moving the cursor eliminates the counter.
            */
         if (i != hedit->position && insert->size) {
-          data = (char *)malloc(insert->size);
+          data = (char *)malloc_fatal(insert->size);
           b_read(insert, data, 0, insert->size);
           if (replace_mode && replace->size) {
             char *data_replace;
-            data_replace = (char *)malloc(replace->size);
+            data_replace = (char *)malloc_fatal(replace->size);
             b_read(replace, data_replace, 0, replace->size);
             he_subcommand(hedit, 0, position, replace->size, data_replace);
             b_clear(replace);
@@ -1653,19 +1708,21 @@ hx_exit_insert_mode:
       if (replace_mode) {
         b_insert(hedit->buffer, position, replace->size);
         b_copy(hedit->buffer, replace, position, 0, replace->size);
-        data_replace = (char *)malloc(replace->size * count);
+        data_replace = (char *)malloc_fatal(replace->size * count);
         replace_size2 =
           b_read(hedit->buffer, data_replace, position, replace->size * count);
-        if (replace_size2 < replace->size * count)
+	if (replace_size2 < 0) {
+	  assert(0); // FIXME: better error handling
+	} else if ((unsigned long)replace_size2 < replace->size * count)
           /* We don't want to eat up more memory than needed.
            */
-          data_replace = (char *)realloc(data_replace, replace_size2);
+          data_replace = (char *)realloc_fatal(data_replace, (unsigned long)replace_size2);
         b_delete(hedit->buffer, position, replace_size2);
         he_subcommand(hedit, 0, position, replace_size2, data_replace);
       }
-      data = (char *)malloc(insert->size);
+      data = (char *)malloc_fatal(insert->size);
       b_read(insert, data, 0, insert->size);
-      data = (char *)realloc(data, insert->size * count);
+      data = (char *)realloc_fatal(data, insert->size * count);
       for (c = insert->size * (count - 1); c > 0; c -= insert->size)
         memcpy(data + c, data, insert->size);
       b_insert(hedit->buffer, position, insert->size * count);
@@ -1678,11 +1735,11 @@ hx_exit_insert_mode:
       } else
         he_refresh_part(hedit, hedit->position, hedit->buffer->size - 1);
     } else {
-      data = (char *)malloc(insert->size);
+      data = (char *)malloc_fatal(insert->size);
       b_read(insert, data, 0, insert->size);
       if (replace_mode)
         if (replace->size) {
-          data_replace = (char *)malloc(replace->size);
+          data_replace = (char *)malloc_fatal(replace->size);
           b_read(replace, data_replace, 0, replace->size);
           he_subcommand(hedit, 0, position, replace->size, data_replace);
         }
@@ -1705,9 +1762,7 @@ hx_exit_insert_mode:
 /* he_insert_mode */
 
   void
-he_scroll_down(hedit, count)
-  struct he_s *hedit;
-  int count;
+he_scroll_down(struct he_s *hedit, int count)
 {
   if (!count) return;
   assert(count > 0);
@@ -1738,9 +1793,7 @@ he_scroll_down(hedit, count)
 /* he_scroll_down */
 
   void
-he_scroll_up(hedit, count)
-  struct he_s *hedit;
-  int count;
+he_scroll_up(struct he_s *hedit, int count)
 {
   if (!count) return;
   assert(count > 0);
@@ -1768,8 +1821,7 @@ he_scroll_up(hedit, count)
 /* he_scroll_up */
 
   int
-he_update_screen(hedit)
-  struct he_s *hedit;
+he_update_screen(struct he_s *hedit)
   /* Update the screen.  If the position of the point is inside the
    * window, the `screen_offset' is adjusted and the screen scrolled.
    * If `refresh.flag' is set, the lines specified by `refresh.first'
@@ -1855,7 +1907,7 @@ he_update_screen(hedit)
     }
   }
   if (hedit->refresh.flag) {
-    /* first we'r gonna check if some of the refresh commands can be
+    /* first we'll check if some of the refresh commands can be
      * combined or omitted.  A cancelled command is marked with a negative
      * `first' value.
      */
@@ -1964,8 +2016,7 @@ he_update_screen(hedit)
 /* he_update_screen */
 
   static int
-he_clear_get(command_mode)
-  int command_mode;
+he_clear_get(int command_mode)
   /* Some users tend to hold down keys like cursor keys or page up/down
    * keys when moving through the buffer.  If the screen update is slower
    * than the repeat rate of the keyboard, `he_clear_get()' takes those
@@ -2003,8 +2054,7 @@ he_clear_get(command_mode)
 /* he_clear_get */
 
   int
-he_mainloop(hedit)
-  struct he_s *hedit;
+he_mainloop(struct he_s *hedit)
 {
   long count = -1;
   int key;
