@@ -3,7 +3,7 @@
  */
 
 /* Copyright (c) 1995,1996 Sascha Demetrio
- * Copyright (c) 2009 Peter Pentchev
+ * Copyright (c) 2009, 2015 Peter Pentchev
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -11,14 +11,14 @@
  * are met:
  * 1. Redistributions of source code must retain the above copyright
  *    notice, this list of conditions and the following disclaimer.
- *    If you modify any part of HEXER and resitribute it, you must add
+ *    If you modify any part of HEXER and redistribute it, you must add
  *    a notice to the `README' file and the modified source files containing
  *    information about the  changes you made.  I do not want to take
  *    credit or be blamed for your modifications.
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- *    If you modify any part of HEXER and resitribute it in binary form,
+ *    If you modify any part of HEXER and redistribute it in binary form,
  *    you must supply a `README' file containing information about the
  *    changes you made.
  * 3. The name of the developer may not be used to endorse or promote
@@ -46,6 +46,7 @@
 
 #include "config.h"
 
+#include <alloca.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -66,16 +67,16 @@
 #include "regex.h"
 #include "set.h"
 #include "tio.h"
+#include "util.h"
 
 #define EXH_DEFAULT_SHELL "/bin/sh"
-#define EXH_DEFAULT_PAGER "more"
 #define EXIT_EXEC_FAILED 27
 
 #ifndef ERESTARTSYS
 #define ERESTARTSYS EINTR
 #endif
 
-char *exh_initialize[] = {
+const char *exh_initialize[] = {
   ":map b :?\\<[~HAT ^V^I]^M",
   ":map w :/\\<[~HAT ^V^I]^M",
   ":map e :/[~HAT ^V^I]\\>^M",
@@ -89,14 +90,14 @@ char *exh_initialize[] = {
 };
 
   static int
-exh_shell_command(char *command, int pager_f)
+exh_shell_command(const char * const command, int pager_f)
                     /* command to be executed by the shell specified by
                      * the "SHELL" environment variable.  the default shell
                      * is `EXH_DEFAULT_SHELL'. */
                     /* if `pager != 0' the output is piped into the
                      * pager specified by the "PAGER" environment.  the
-                     * default pager is `EXH_DEFAULT_PAGER'. */
-  /* we won't use the "-c"-switch (available for most shells), instead we're
+                     * default pager is `HE_DEFAULT_PAGER'. */
+  /* we won't use the "-c"-switch (available for most shells), instead we'll
    * gonna pipe the command into the shell.
    */
 {
@@ -110,10 +111,10 @@ exh_shell_command(char *command, int pager_f)
   tio_suspend();
   tio_clear();
   tio_flush();
-  if (!(*shell = getenv("SHELL"))) *shell = EXH_DEFAULT_SHELL;
-  *shell = strdup(*shell);
-  if (!(*pager = getenv("PAGER"))) *pager = EXH_DEFAULT_PAGER;
-  *pager = strdup(*pager);
+  if (!(*shell = getenv("SHELL"))) *shell = strdup_fatal(EXH_DEFAULT_SHELL);
+  *shell = strdup_fatal(*shell);
+  if (!(*pager = getenv("PAGER"))) *pager = strdup_fatal(HE_DEFAULT_PAGER);
+  *pager = strdup_fatal(*pager);
   /* break `*shell' and `*pager' down into whitespace separated
    * substrings.  it is *not* possible to mask whitespace characters in any
    * way. */
@@ -142,8 +143,8 @@ exh_shell_command(char *command, int pager_f)
         close(pipe2[0]);
         close(pipe2[1]);
         /* send the command to the shell */
-        if (write(pipe1[1], command, strlen(command)) != strlen(command) ||
-	    write(pipe1[1], "\n", 1) != 1) {
+        if (write_buf(pipe1[1], command, strlen(command)) < 0 ||
+	    write_buf(pipe1[1], "\n", 1) < 0) {
 	  close(pipe1[1]);
 	  he_message(1, "couldn't send the `%s' command to the pager", command);
 	  close(pipe2[0]);
@@ -196,8 +197,8 @@ exh_shell_command(char *command, int pager_f)
         exit(EXIT_EXEC_FAILED);
       }
     } else { /* !pager_f */
-      if (write(pipe1[1], command, strlen(command)) != strlen(command) ||
-	  write(pipe1[1], "\n", 1) != 1) {
+      if (write_buf(pipe1[1], command, strlen(command)) < 0 ||
+	  write_buf(pipe1[1], "\n", 1) < 0) {
 	close(pipe1[1]);
 	he_message(1, "couldn't send command `%s' to pager", command);
       } else {
@@ -264,8 +265,8 @@ exh_subshell(void)
   tio_suspend();
   tio_clear();
   tio_flush();
-  if (!(*shell = getenv("SHELL"))) *shell = EXH_DEFAULT_SHELL;
-  *shell = strdup(*shell);
+  if (!(*shell = getenv("SHELL"))) *shell = strdup_fatal(EXH_DEFAULT_SHELL);
+  *shell = strdup_fatal(*shell);
   /* break `*shell' down into whitespace separated substrings.
    * it is *not* possible to mask whitespace characters in any way. */
   for (i = 0; *shell[i]; shell[++i] = p) {
@@ -296,17 +297,15 @@ exh_subshell(void)
 }
 /* exh_subshell */
 
-  char *
-exh_skip_expression(exp, separator)
-  char *exp;
-  char separator;
-  /* returns a pointer to the first unmasked occurence of the separator
+  const char *
+exh_skip_expression(const char *exp, char separator)
+  /* returns a pointer to the first unmasked occurrence of the separator
    * `separator'; separators within a range or prefixed by `\' are ignored.
    * if no separator is found, 0 is returned.
    */
 {
-  char *p = exp;
-  char *s = 0;
+  const char *p = exp;
+  const char *s = 0;
 
   for (;; ++p) {
     while (*p && *p != separator && *p != '\\' && *p != '[') ++p;
@@ -332,16 +331,14 @@ exh_skip_expression(exp, separator)
 }
 /* exh_skip_expression */
 
-  char *
-exh_skip_replace(exp, separator)
-  char *exp;
-  char separator;
+  const char *
+exh_skip_replace(const char *exp, char separator)
   /* similar to `exh_skip_expression()', but ignores the special meaning of
    * brackets.
    */
 {
-  char *p = exp;
-  char *s = 0;
+  const char *p = exp;
+  const char *s = 0;
 
   for (;; ++p) {
     while (*p && *p != separator && *p != '\\') ++p;
@@ -358,10 +355,10 @@ exh_skip_replace(exp, separator)
 }
 /* exh_skip_replace */
 
-  static char *
-exh_get_number(char *exp, long *number)
+  static const char *
+exh_get_number(const char *exp, unsigned long *number)
 {
-  char *p;
+  const char *p;
 
   switch (*exp) {
     case '0':  /* octal or hex number */
@@ -412,21 +409,21 @@ exh_get_number(char *exp, long *number)
 }
 /* exh_get_number */
 
-  static char *
-exh_get_address(struct he_s *hedit, char *exp, long *address)
+  static const char *
+exh_get_address(struct he_s *hedit, const char *exp, unsigned long *address)
   /* evaluate the specified address and store it in `*address'.  on success
    * a skip-pointer for the expression is returned, else (if `exp' is not
    * a valid address or expression) 0 is returned.
    */
 {
-  char *p = exp, *q;
+  const char *p = exp, *q;
   char *rs;
   long rl, ml;
   int direction;
   char separator;
-  long n;
+  unsigned long n;
   long positive_f;
-  long offset;
+  unsigned long offset;
   int eox_f = 0;
   long old_position = hedit->position;
 
@@ -441,8 +438,14 @@ exh_get_address(struct he_s *hedit, char *exp, long *address)
         /* fall through */
       case '/':  /* forward search */
         p = exh_skip_expression(q = p + 1, separator);
-        if (p) *p++ = 0; else p = exp + strlen(exp);
-        *address = he_search(hedit, q, "", direction, 0, 1, -1, &rs, &rl, &ml);
+	size_t len = p? (size_t)(p - q): strlen(exp);
+	if (p)
+		p++;
+	else
+		p = exp + len;
+	char *needle = alloca(len + 1);
+	snprintf(needle, len + 1, "%s", q);
+        *address = he_search(hedit, needle, "", direction, 0, 1, -1, &rs, &rl, &ml);
         switch (*address) {
           case -2:  /* invalid expression */
             assert(rx_error);
@@ -467,19 +470,20 @@ exh_get_address(struct he_s *hedit, char *exp, long *address)
       case '5': case '6': case '7': case '8': case '9':
         p = exh_get_number(p, address);
         if (*address > (n = hedit->buffer->size)) {
-          he_message(0, "only 0x%lx (%li) bytes", n, n);
+          he_message(0, "only 0x%lx (%lu) bytes", n, n);
           p = 0;
         } else
           hedit->position = *address;
         break;
       case '#':  /* line number */
         p = exh_get_number(p + 1, &n);
-        if ((*address = b_goto_line(hedit->buffer, n)) < 0) {
+	long naddr = b_goto_line(hedit->buffer, n);
+	if (naddr < 0) {
           n = b_no_lines(hedit->buffer);
-          he_message(0, "only 0x%lx (%li) lines", n, n);
+          he_message(0, "only 0x%lx (%lu) lines", n, n);
           p = 0;
         } else
-          hedit->position = *address;
+          hedit->position = *address = naddr;
         break;
       case '.':  /* current position */
         p = exp + 1;
@@ -499,13 +503,14 @@ exh_get_address(struct he_s *hedit, char *exp, long *address)
           p = 0;
           break;
         }
+	long naddress = *address;
         if (positive_f)
-          *address += offset;
+          naddress += offset;
         else
-          *address -= offset;
-        if (*address < 0) *address = 0;
-        if (*address > hedit->buffer->size) *address = hedit->buffer->size;
-        hedit->position = *address;
+          naddress -= offset;
+        if (naddress < 0) naddress = 0;
+        if ((unsigned long)naddress > hedit->buffer->size) naddress = hedit->buffer->size;
+	hedit->position = *address = naddress;
         break;
       default:
         eox_f = 1;
@@ -518,9 +523,7 @@ exh_get_address(struct he_s *hedit, char *exp, long *address)
 /* exh_get_address */
 
   int
-exh_command(hedit, cmd)
-  struct he_s *hedit;
-  char *cmd;
+exh_command(struct he_s * const hedit, const char * const cmd)
   /* execute the exh-command `cmd' on `hedit'.  multiple commands may be
    * specified (separated by semicolons).
    * note:  if any of the commands changes the current buffer, all successive
@@ -531,18 +534,20 @@ exh_command(hedit, cmd)
    */
 {
   char command[256];
-  char *p;
-  long begin = 0, end = 0;
+  const char *p;
+  unsigned long begin = 0, end = 0;
   int i, j, k;
   int terminal_match_f = -1;
-  char *skip;
+  const char *skip;
   struct buffer_s *current = current_buffer;
 
-  p = (char *)alloca(strlen(cmd) + 1);
-  strcpy(p, cmd);
+  size_t psize = strlen(cmd) + 1;
+  char *pbuf = (char *)alloca(psize);
+  snprintf(pbuf, psize, "%s", cmd);
   /* remove trailing whitespace */
-  for (i = strlen(p) - !!*p; i && (p[i] == ' ' || p[i] == '\t'); --i);
-  if (*p) p[i + 1] = 0;
+  for (i = psize - 1 - !!*pbuf; i && (pbuf[i] == ' ' || pbuf[i] == '\t'); --i);
+  if (*pbuf) pbuf[i + 1] = 0;
+  p = pbuf;
 check_terminal:
   while (*p == ':') ++p;
   /* remove leading whitespace */
@@ -585,7 +590,7 @@ check_terminal:
     } else
       end = begin;
     if (begin > end) {
-      if (he_query_yn(1, "@Abfirst address exceeds the second@~ - swap") > 0)
+      if (he_query_yn(1, "@Abfirst address succeeds the second@~ - swap") > 0)
 	begin ^= end, end ^= begin, begin ^= end;
       else
 	goto exit_exh_command;
@@ -598,7 +603,7 @@ check_terminal:
   command[i] = 0;
 
   if (!*command) {
-    /* no command.  go to the given address, if two adresses were selected,
+    /* no command.  go to the given address, if two addresses were selected,
      * select the specified area.  */
     if (end > begin) { /* area */
       he_select(hedit, begin, end);
@@ -634,7 +639,7 @@ check_terminal:
 exh_execute:
   if (begin == 0 && end == 0) {
     begin = exh_commands[i].whole_f ? 0 : hedit->position;
-    end = exh_commands[i].whole_f ? hedit->buffer->size - 1 : hedit->position;
+    end = exh_commands[i].whole_f ? hedit->buffer->size - 1 : (unsigned long)hedit->position;
   }
   while (*p == ' ' || *p == '\t') ++p;
   skip = exh_commands[i].cmd(hedit, p, begin, end);
@@ -649,7 +654,7 @@ exit_exh_command:
  */
 
   static char **
-exh_cpl_file_list(char *prefix)
+exh_cpl_file_list(const char * const prefix)
 {
   int i;
   DIR *dp;
@@ -670,10 +675,10 @@ exh_cpl_file_list(char *prefix)
   while (p > dirname) if (*p != '/') --p; else break;
   if (p != dirname || *dirname == '/') ++p;
   *p = 0;
-  /* check if the `prefix' starts with a "~user/"-aberv..
+  /* check if the `prefix' starts with a "~user/"-abbrev..
    */
   if (*prefix == '~') {
-    user = strdup(prefix + 1);
+    user = strdup_fatal(prefix + 1);
     for (p = user; *p && *p != '/'; ++p);
     if (!*p) {
       /* hmm... the username `user' is possibly incomplete.  check how many
@@ -685,13 +690,13 @@ exh_cpl_file_list(char *prefix)
       setpwent();
       /* make the list.
        */
-      *(list = (char **)malloc(sizeof(char *) * ++i)) = 0;
+      *(list = (char **)malloc_fatal(sizeof(char *) * ++i)) = 0;
       for (i = 0; (pe = getpwent());)
         if (*user ? !strncmp(pe->pw_name, user, strlen(user)) : 1) {
           *d_name = '~';
           strcpy(d_name + 1, pe->pw_name);
           strcat(d_name, "/");
-          list[i++] = strdup(d_name);
+          list[i++] = strdup_fatal(d_name);
         }
       list[i] = 0;
       endpwent();
@@ -708,7 +713,7 @@ exh_cpl_file_list(char *prefix)
         goto empty_list;
       }
     } else {
-      int uid = getuid();
+      uid_t uid = getuid();
       setpwent();
       for (; (pe = getpwent());) if (pe->pw_uid == uid) break;
       if (!pe) {
@@ -731,20 +736,20 @@ exh_cpl_file_list(char *prefix)
     strcat(d_name, de->d_name);
     if (!strncmp(d_name, prefix, plen)) ++i;
   }
-  list = (char **)malloc(sizeof(char *) * ++i);
+  list = (char **)malloc_fatal(sizeof(char *) * ++i);
   for (rewinddir(dp), i = 0; (de = readdir(dp));) {
     struct stat st;
     char *rp;
     strcpy(d_name, dirname);
     strcat(d_name, de->d_name);
     if (!strncmp(d_name, prefix, plen)) {
-      rp = (char *)malloc(strlen(path) + strlen(de->d_name) + 4);
+      rp = (char *)malloc_fatal(strlen(path) + strlen(de->d_name) + 4);
       strcpy(rp, path);
       strcat(rp, "/");
       strcat(rp, de->d_name);
       if (!stat(rp, &st)) if (S_ISDIR(st.st_mode)) strcat(d_name, "/");
       free(rp);
-      list[i++] = strdup(d_name);
+      list[i++] = strdup_fatal(d_name);
     }
   }
   list[i] = 0;
@@ -754,13 +759,13 @@ exit:
   return list;
 empty_list:
   if (user) free(user);
-  *(list = (char **)malloc(sizeof(char *))) = 0;
+  *(list = (char **)malloc_fatal(sizeof(char *))) = 0;
   return list;
 }
 /* exh_cpl_file_list */
 
   static char **
-exh_cpl_command_list(char *prefix)
+exh_cpl_command_list(const char * const prefix)
 {
   int i, j;
   int plen = strlen(prefix);
@@ -772,18 +777,18 @@ exh_cpl_command_list(char *prefix)
         && exh_commands[i].cmd_char
         && (plen ? exh_commands[i].cmd_char == *prefix : 1)) ++j;
   }
-  list = (char **)malloc((j + 1) * sizeof(char *));
+  list = (char **)malloc_fatal((j + 1) * sizeof(char *));
   if (j) {
     for (i = 0, j = 0; exh_commands[i].cmd_name; ++i) {
       if (!strncmp(exh_commands[i].cmd_name, prefix, plen)) {
 	list[j] =
-          (char *)malloc(strlen(exh_commands[i].cmd_name) + 1);
+          (char *)malloc_fatal(strlen(exh_commands[i].cmd_name) + 1);
 	strcpy(list[j++], exh_commands[i].cmd_name);
       }
       if (exh_commands[i].cmd_name[0] != exh_commands[i].cmd_char
           && exh_commands[i].cmd_char
           && (plen ? exh_commands[i].cmd_char == *prefix : 1)) {
-        list[j] = (char *)malloc(2);
+        list[j] = (char *)malloc_fatal(2);
         list[j][0] = exh_commands[i].cmd_char;
         list[j++][1] = 0;
       }
@@ -796,7 +801,7 @@ exh_cpl_command_list(char *prefix)
 /* exh_cpl_command_list */
 
   static char **
-exh_cpl_buffer_list(char *prefix)
+exh_cpl_buffer_list(const char * const prefix)
 {
   struct buffer_s *i;
   char **list;
@@ -805,11 +810,11 @@ exh_cpl_buffer_list(char *prefix)
 
   for (j = 0, i = buffer_list; i; i = i->next)
     if (!strncmp(i->hedit->buffer_name, prefix, plen)) ++j;
-  list = (char **)malloc((j + 1) * sizeof(char *));
+  list = (char **)malloc_fatal((j + 1) * sizeof(char *));
   if (j) {
     for (j = 0, i = buffer_list; i; i = i->next)
       if (!strncmp(i->hedit->buffer_name, prefix, plen)) {
-	list[j] = (char *)malloc(strlen(i->hedit->buffer_name) + 1);
+	list[j] = (char *)malloc_fatal(strlen(i->hedit->buffer_name) + 1);
 	strcpy(list[j++], i->hedit->buffer_name);
       }
     list[j] = 0;
@@ -820,7 +825,7 @@ exh_cpl_buffer_list(char *prefix)
 /* exh_cpl_buffer_list */
 
   static char **
-exh_cpl_file_and_buffer_list(char *prefix)
+exh_cpl_file_and_buffer_list(const char * const prefix)
 {
   char **list1 = exh_cpl_file_list(prefix);
   char **list2 = exh_cpl_buffer_list(prefix);
@@ -835,7 +840,7 @@ exh_cpl_file_and_buffer_list(char *prefix)
   for (i = 0; list1[i]; ++i);
   for (j = k = 0; list2[j]; ++j) if (list2[j] != (char *)1) ++k;
   
-  list1 = (char **)realloc(list1, (i + k + 1) * sizeof(char *));
+  list1 = (char **)realloc_fatal(list1, (i + k + 1) * sizeof(char *));
   for (j = k = i; list2[j - i]; ++j)
     if (list2[j - i] != (char *)1) {
       list1[k] = list2[j - i];
@@ -848,7 +853,7 @@ exh_cpl_file_and_buffer_list(char *prefix)
 /* exh_cpl_file_and_buffer_list */
 
   static char **
-exh_cpl_option_list(char *prefix)
+exh_cpl_option_list(const char *prefix)
 {
   char **list, **i;
   int j, k;
@@ -858,7 +863,7 @@ exh_cpl_option_list(char *prefix)
   i = list = s_option_list(prefix, no_f);
   if (no_f) {
     while (*i) {
-      *i = (char *)realloc(*i, k = strlen(*i) + 2);
+      *i = (char *)realloc_fatal(*i, k = strlen(*i) + 2);
       for (j = k; j >= 0; --j) (*i)[j] = (*i)[j - 2];
       **i = 'n', (*i)[1] = 'o';
       ++i;
@@ -869,11 +874,7 @@ exh_cpl_option_list(char *prefix)
 /* exh_cpl_option_list */
 
   char **
-exh_completer(prefix, command, line, context)
-  char *prefix;
-  char *command;
-  char *line; /* unused */
-  int context;
+exh_completer(const char * const prefix, const char * const command, char * const line __unused, const int context)
   /* we're looking for completions of `prefix'; `command' is the
    * command-string that `prefix' is the argument of; `line' is the
    * complete line before the cursor.
@@ -882,7 +883,7 @@ exh_completer(prefix, command, line, context)
 {
   int expect = -1;
   int i, j, k;
-  char **(*completer[16])(char *);
+  char **(*completer[16])(const char *);
   completer[0] = exh_cpl_command_list;
   completer[1] = exh_cpl_file_list;
   completer[2] = exh_cpl_buffer_list;
@@ -929,7 +930,7 @@ exh_completer(prefix, command, line, context)
   }
 exit_exh_competer:
   if (expect < 0) {
-    char **list = (char **)malloc(sizeof(char *));
+    char **list = (char **)malloc_fatal(sizeof(char *));
     *list = 0;
     return list;
   } else
