@@ -2,7 +2,7 @@
  */
 
 /* Copyright (c) 1995,1996 Sascha Demetrio
- * Copyright (c) 2009, 2010 Peter Pentchev
+ * Copyright (c) 2009, 2010, 2015 Peter Pentchev
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -10,14 +10,14 @@
  * are met:
  * 1. Redistributions of source code must retain the above copyright
  *    notice, this list of conditions and the following disclaimer.
- *    If you modify any part of HEXER and resitribute it, you must add
+ *    If you modify any part of HEXER and redistribute it, you must add
  *    a notice to the `README' file and the modified source files containing
  *    information about the  changes you made.  I do not want to take
  *    credit or be blamed for your modifications.
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- *    If you modify any part of HEXER and resitribute it in binary form,
+ *    If you modify any part of HEXER and redistribute it in binary form,
  *    you must supply a `README' file containing information about the
  *    changes you made.
  * 3. The name of the developer may not be used to endorse or promote
@@ -45,6 +45,7 @@
 
 #include "config.h"
 
+#include <alloca.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -77,34 +78,22 @@ struct buffer_s *current_buffer;
 struct buffer_s *buffer_list = 0;
 struct he_message_s *he_messages;
 char *alternate_buffer;
-char *he_pagerprg;
+const char *he_pagerprg;
 
-#if USE_STDARG
   void
-he_message(int beep, char *fmt, ...)
-#else
-  void
-he_message(beep, fmt, va_alist)
-  int beep;
-  char *fmt;
-  va_dcl
-#endif
+he_message(const int beep, const char * const fmt, ...)
 {
   va_list ap;
   struct he_message_s *m;
   /* int length; */
 
-#if USE_STDARG
   va_start(ap, fmt);
-#else
-  va_start(ap);
-#endif
   /* length = tio_nprintf(fmt, ap); */
-  m = (struct he_message_s *)malloc(sizeof(struct he_message_s));
+  m = (struct he_message_s *)malloc_fatal(sizeof(struct he_message_s));
   m->next = he_messages;
   m->beep = beep;
-  /* m->message = (char *)malloc(length + 1); */
-  m->message = (char *)malloc(512); /* FIXME */
+  /* m->message = (char *)malloc_fatal(length + 1); */
+  m->message = (char *)malloc_fatal(512); /* FIXME */
   vsprintf(m->message, fmt, ap);
   he_messages = m;
   va_end(ap);
@@ -167,9 +156,9 @@ action_mapmagic(int current_value)
 /* action_mapmagic */
 
 static const struct hexer_options_s {
-  char *option;
+  const char *option;
   enum s_option_e type;
-  char *default_value;
+  const char *default_value;
   set_fn action;
 } hexer_options[] = {
   { "ascii", S_BOOL, "true", (set_fn)action_ascii },
@@ -188,9 +177,7 @@ static const struct hexer_options_s {
  */
 
   int
-he_open_buffer(name, path)
-  char *name;
-  char *path;
+he_open_buffer(const char * const name, const char * const path)
 {
   struct buffer_s *buffer;
   int no_file_f = 0, read_only = 0;
@@ -219,13 +206,13 @@ he_open_buffer(name, path)
         return -1;
       }
   }
-  *(buffer = (struct buffer_s *)malloc(sizeof(struct buffer_s))) = NO_BUFFER;
-  buffer->hedit = (struct he_s *)malloc(sizeof(struct he_s));
+  *(buffer = (struct buffer_s *)malloc_fatal(sizeof(struct buffer_s))) = NO_BUFFER;
+  buffer->hedit = (struct he_s *)malloc_fatal(sizeof(struct he_s));
   memset(buffer->hedit, 0, sizeof (struct he_s));
   buffer->hedit->begin_selection = -1;
   buffer->hedit->end_selection = -1;
   buffer->hedit->insert_position = -1;
-  buffer->hedit->buffer_name = strdup(name);
+  buffer->hedit->buffer_name = strdup_fatal(name);
   if (path && !no_file_f) {
     if (!(fp = fopen(path, "r"))) {
       he_message(1, "`%s': @Ab%s@~", path, strerror(errno));
@@ -241,13 +228,13 @@ he_open_buffer(name, path)
     buffer->visited_f = 1;
   }
   if (path) {
-    buffer->path = strdup(path);
+    buffer->path = strdup_fatal(path);
     if (!getcwd(cwd, PATH_MAX)) {
       he_message(0, "@Abcan't get cwd: %s@~", strerror(errno));
-      buffer->fullpath = strdup(path);
+      buffer->fullpath = strdup_fatal(path);
     } else {
       buffer->fullpath =
-        (char *)malloc(strlen(path) + strlen(cwd) + 2);
+        (char *)malloc_fatal(strlen(path) + strlen(cwd) + 2);
       sprintf(buffer->fullpath, "%s/%s", cwd, path);
     }
   }
@@ -263,10 +250,9 @@ he_open_buffer(name, path)
 /* he_open_buffer */
 
   int
-he_select_buffer_(buffer)
-  struct buffer_s *buffer;
+he_select_buffer_(const struct buffer_s * const buffer)
   /* Set `current_buffer' to `buffer'.  The file for `buffer' is loaded if
-   * nessecarry.
+   * necessary.
    */
 {
   struct buffer_s *i;
@@ -287,7 +273,7 @@ he_select_buffer_(buffer)
     }
     /* check out if we can open a swapfile */
     i->hedit->swapfile =
-      (char *)malloc(strlen(hexer_ext) + strlen(i->path) + 1);
+      (char *)malloc_fatal(strlen(hexer_ext) + strlen(i->path) + 1);
     strcpy(i->hedit->swapfile, i->path);
     strcat(i->hedit->swapfile, hexer_ext);
     if (access(i->hedit->swapfile, R_OK)) {
@@ -305,7 +291,7 @@ he_select_buffer_(buffer)
       he_message(1, "@Abwarning: swapfile@~ `%s' @Abexists@~",
                  i->hedit->swapfile);
       i->hedit->swapfile =
-        (char *)realloc(i->hedit->swapfile,
+        (char *)realloc_fatal(i->hedit->swapfile,
                       strlen(i->hedit->swapfile) + strlen(swap_template) + 1);
         strcat(i->hedit->swapfile, swap_template);
       if ((swapfd = mkstemp(i->hedit->swapfile)) < 0)
@@ -324,7 +310,7 @@ he_select_buffer_(buffer)
       len = 6 + strlen(HEXER_VERSION) + 1 + strlen(i->fullpath) + 1 + 4;
       buf = alloca(len + 1);
       if (snprintf(buf, len + 1, "hexer %s\n%s\n%c%c%c%c",
-	  HEXER_VERSION, i->fullpath, 0, 0, 0, 0) != len ||
+	  HEXER_VERSION, i->fullpath, 0, 0, 0, 0) != (int)len ||
 	  fwrite(buf, 1, len, i->hedit->undo.swapfile) != len ||
 	  fflush(i->hedit->undo.swapfile) == EOF) {
 	/* TODO: some kind of error output */
@@ -341,8 +327,7 @@ he_select_buffer_(buffer)
 /* he_select_buffer_ */
 
   int
-he_select_buffer(name)
-  char *name;
+he_select_buffer(const char * const name)
 {
   struct buffer_s *i;
 
@@ -354,7 +339,7 @@ he_select_buffer(name)
 /* he_select_buffer */
 
   int
-he_alternate_buffer()
+he_alternate_buffer(void)
 {
   char *ab = alternate_buffer;
 
@@ -368,8 +353,7 @@ he_alternate_buffer()
 /* he_alternate_buffer */
 
   int
-he_set_buffer_readonly(name)
-  char *name;
+he_set_buffer_readonly(const char * const name)
   /* Return values:
    * -1: no buffer named `name'
    * 0:  ok
@@ -386,12 +370,11 @@ he_set_buffer_readonly(name)
 /* he_set_buffer_readonly */
 
   int
-he_buffer_readonly(name)
-  char *name;
+he_buffer_readonly(char *name)
   /* Return values:
    * -1: no buffer named `name'
-   * 0:  buffer is readwrite
-   * 1:  buffer is readonly
+   * 0:  buffer is read/write
+   * 1:  buffer is read-only
    */
 {
   struct buffer_s *i;
@@ -404,8 +387,7 @@ he_buffer_readonly(name)
 /* he_buffer_readonly */
 
   int
-he_buffer_modified(name)
-  char *name;
+he_buffer_modified(char *name)
   /* Return values:
    * -1: no buffer named `name'
    * 0:  buffer saved
@@ -422,8 +404,7 @@ he_buffer_modified(name)
 /* he_buffer_modified */
 
   int
-he_close_buffer(name)
-  char *name;
+he_close_buffer(const char * const name)
   /* Close the buffer named `name'. If `name == 0', the current buffer
    * is closed.  The return value is 0 if all goes well, 1 if the named
    * buffer doesn't exist and -1 if the `buffer_list' is empty.
@@ -485,8 +466,7 @@ he_close_buffer(name)
  */
 
   void
-he_status_message(verbose)
-  int verbose;
+he_status_message(int verbose)
   /* display name and size of the current buffer.  if `verbose' is set,
    * the current position is also displayed.
    */
@@ -519,10 +499,7 @@ he_status_message(verbose)
 /* he_status_message */
 
   char *
-he_query_command(prompt, dfl, context)
-  char *prompt;
-  char *dfl;
-  int context;
+he_query_command(const char *prompt, const char *dfl, int context)
   /* Convention:
    * `context == 0': exh-command;
    * `context == 1': shell-command;
@@ -536,26 +513,14 @@ he_query_command(prompt, dfl, context)
 }
 /* he_query_command */
 
-#if USE_STDARG
   int
-he_query_yn(int dfl, char *fmt, ...)
-#else
-  int
-he_query_yn(dfl, fmt, va_alist)
-  int dfl;
-  char *fmt;
-  va_dcl
-#endif
+he_query_yn(int dfl, const char *fmt, ...)
 {
   va_list ap;
   int key;
   int choice;
 
-#if USE_STDARG
   va_start(ap, fmt);
-#else
-  va_start(ap);
-#endif
   tio_keypad(0);
 restart:
   tio_goto_line(hx_lines - 1);
@@ -650,44 +615,34 @@ rxwrap_tell(void)
 /* rxwrap_tell */
 
   long
-he_search(hedit, exp, replace, direction, wrap, increment, end,
-          replace_str, replace_len, match_len)
-  struct he_s *hedit;
-  char *exp;
+he_search(struct he_s *hedit, const char *exp, const char *replace, int direction, int wrap, int increment, long end,
+          char **replace_str, long *replace_len, long *match_len)
     /* regular expression.
      */
-  char *replace;
     /* replace template.  the replace template may contain back references to
      * the regular expression (`\0', ... `\9').
      */
-  int direction;
     /* `direction >= 0': forward search.
      * `direction < 0': reverse search.
      */
-  int wrap;
-    /* if `wrap' is set, the search continues at the top of the buffer/file
-     * if the bottom has been (or vice versa, depending on `direction').
+    /* if `wrap' is set, the search continues from the top of the buffer/file
+     * once the bottom has been passed (or vice versa, depending on `direction').
      */
-  int increment;
     /* if `increment' is set, the search starts at `hedit->position + 1'
      * rather than at `hedit->position'.  if the direction is set to reverse
-     * search, the `increment'-flag has no effect.
+     * search, the `increment' flag has no effect.
      */
-  long end;
     /* if `wrap' is not set and `end' is not negative, the search ends at
      * position `end'.
      */
-  char **replace_str;
     /* if `replace_str' is non-zero and a match was found, the replace
      * string generated from `replace' will be copied to `*replace_str'.
      * the memory for that replace string will be allocated via `malloc()'.
-     * NOTE: the replace string wont be terminated with a null-character
+     * NOTE: the replace string won't be terminated with a null character
      *   since it may contain null characters.
      */
-  long *replace_len;
     /* the length of the replace sting is written to `*replace_len'.
      */
-  long *match_len;
     /* the length of the match is written to `*replace_len'.
      */
   /* RETURN VALUE:  if a match was found, the position of the match is
@@ -731,10 +686,7 @@ he_search(hedit, exp, replace, direction, wrap, increment, end,
 /* he_search */
 
   void
-he_search_command(hedit, exp, dir)
-  struct he_s *hedit;
-  char *exp;
-  int dir;
+he_search_command(struct he_s *hedit, char *exp, int dir)
 {
   long position;
   char *rs;
@@ -780,20 +732,20 @@ he_refresh(void)
 /* he_refresh */
 
   void
-hexer_version()
+hexer_version(void)
 {
   he_message(0, "@AbHexer@~ version @U%s@u", HEXER_VERSION);
 }
 /* hexer_version */
 
   void
-hexer_init()
+hexer_init(void)
   /* this function is called by `process_args()' (main.c) before executing
    * the commands given at the command line.
    */
 {
   int i;
-  char *hexerinit, *home;
+  const char *hexerinit, *home;
   char path[1024];
   char line[1024];
   FILE *fp;
@@ -837,7 +789,7 @@ hexer_init()
 /* hexer_init */
 
   int
-hexer()
+hexer(void)
 {
   int key;
   char *cmd;
