@@ -5,7 +5,7 @@
  */
 
 /* Copyright (c) 1995,1996 Sascha Demetrio
- * Copyright (c) 2009 Peter Pentchev
+ * Copyright (c) 2009, 2015 Peter Pentchev
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -13,14 +13,14 @@
  * are met:
  * 1. Redistributions of source code must retain the above copyright
  *    notice, this list of conditions and the following disclaimer.
- *    If you modify any part of HEXER and resitribute it, you must add
+ *    If you modify any part of HEXER and redistribute it, you must add
  *    a notice to the `README' file and the modified source files containing
  *    information about the  changes you made.  I do not want to take
  *    credit or be blamed for your modifications.
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- *    If you modify any part of HEXER and resitribute it in binary form,
+ *    If you modify any part of HEXER and redistribute it in binary form,
  *    you must supply a `README' file containing information about the
  *    changes you made.
  * 3. The name of the developer may not be used to endorse or promote
@@ -59,10 +59,6 @@
 #include "tio.h"
 #include "util.h"
 
-#ifndef USE_PROTOTYPES
-#define USE_PROTOTYPES 1
-#endif
-
 #define LINE_MAXLEN 8192        /* maximum length of the line */
 #define RL_MAX_CONTEXTS 16      /* maximum number of history contexts */
 
@@ -70,11 +66,7 @@
                                  * partially scrolled out to the left */
 #define RL_RIGHT_MORE "@Ab>@~"  /* character displayed if the line is
                                  * partially scrolled out to the right */
-#if ('\e' == 'e')
-#define RL_ESC ((char)27)
-#else
-#define RL_ESC ('\e')
-#endif
+#define RL_ESC ('\033')
 
 /* strings:
  * `readline()' returns strings of characters.  special characters (like
@@ -89,19 +81,19 @@
  * of 254 (== HXKEY_BIAS) is used for the other keys).
  */
 
-static char *rl_prompt;    /* the prompt */
+static const char *rl_prompt;    /* the prompt */
 static int rl_prompt_len;  /* the length of the prompt in characters */
-static char *rl_default;   /* the default answer */
+static const char *rl_default;   /* the default answer */
 
 static char rl_line[LINE_MAXLEN];
                            /* buffer for the current line */
 static char rl_vline[LINE_MAXLEN];
                            /* buffer for the "visible" line */
-static int rl_position;    /* position of the cursor.  the position of the
+static size_t rl_position; /* position of the cursor.  the position of the
                             * cursor is number of the character under the
                             * cursor.  note that escape-sequences are
                             * considered to be a single character. */
-static int rl_offset;      /* number of characters scrolled out to the
+static size_t rl_offset;   /* number of characters scrolled out to the
                             * left, so the first character displayed on
                             * screen is `rl_vline[rl_offset]'. */
 
@@ -110,11 +102,7 @@ struct rl_line_s {
   char *vline;
 } rl = { rl_line, rl_vline };
 
-#if USE_PROTOTYPES
-char **(*completer)(char *prefix, char *command, char *line, int context);
-#else
-char **(*completer)();
-#endif
+char **(*completer)(const char *prefix, const char *command, char *line, int context);
   /* Pointer to the completer function.  If `completer == 0', completion
    * is disabled.
    */
@@ -167,11 +155,11 @@ rl_history_init(int max)
 
   for (i = 0; i < RL_MAX_CONTEXTS; ++i) {
     rl_history[i] =
-      (struct rl_line_s *)malloc(max * sizeof(struct rl_line_s));
+      (struct rl_line_s *)malloc_fatal(max * sizeof(struct rl_line_s));
     for (j = 0; j < max; ++j) {
-      rl_history[i][j].line = (char *)malloc(1);
+      rl_history[i][j].line = (char *)malloc_fatal(1);
       *rl_history[i][j].line = 0;
-      rl_history[i][j].vline = (char *)malloc(1);
+      rl_history[i][j].vline = (char *)malloc_fatal(1);
       *rl_history[i][j].vline = 0;
     }
     rl_history_c[i] = 1;
@@ -201,12 +189,12 @@ rl_history_add(struct rl_line_s line)
       rl_history[rl_current_context][i] =
         rl_history[rl_current_context][i + 1];
     rl_history[rl_current_context][i].line =
-      (char *)malloc(strlen(line.line) + 1);
+      (char *)malloc_fatal(strlen(line.line) + 1);
   } else {
     i = rl_history_c[rl_current_context] - 1;
     ++rl_history_c[rl_current_context];
     rl_history[rl_current_context][i].line =
-      (char *)realloc(rl_history[rl_current_context][i].line,
+      (char *)realloc_fatal(rl_history[rl_current_context][i].line,
                       strlen(line.line) + 1);
   }
   strcpy(rl_history[rl_current_context][i].line, line.line);
@@ -259,7 +247,7 @@ rl_history_set(struct rl_line_s line)
    */
 {
   rl_history[rl_current_context][rl_index].line =
-    (char *)realloc(rl_history[rl_current_context][rl_index].line,
+    (char *)realloc_fatal(rl_history[rl_current_context][rl_index].line,
                     strlen(line.line) + 1);
   strcpy(rl_history[rl_current_context][rl_index].line, line.line);
 }
@@ -334,13 +322,13 @@ exit_rl_query_yn:
 }
 /* rl_query_yn */
 
-  static int
+  static size_t
 rl_get_position(void)
   /* return the position of the cursor in `rl.line'.
    */
 {
-  int i;
-  int position;
+  size_t i;
+  size_t position;
 
   for (i = position = 0; rl.line[i] && position < rl_position; ++i) {
     ++position;
@@ -350,14 +338,14 @@ rl_get_position(void)
 }
 /* rl_get_position */
 
-  static int
+  static size_t
 rl_get_vposition(void)
   /* return the position of the cursor in `rl.vline'.
    */
 {
-  int i;
-  int position;
-  int vposition;
+  size_t i;
+  size_t position;
+  size_t vposition;
   int key;
 
   for (i = position = vposition = 0;
@@ -386,14 +374,14 @@ rl_get_vposition(void)
 }
 /* rl_get_position */
 
-  static int
+  static size_t
 rl_get_length(struct rl_line_s *rrl)
   /* return the number of logical characters in `rrl->line'.  escape sequences
    * are counted as a single character.
    */
 {
-  int i;
-  int length;
+  size_t i;
+  size_t length;
 
   for (i = length = 0; rrl->line[i]; ++i, ++length)
     if (rrl->line[i] == RL_ESC) ++i;
@@ -410,7 +398,7 @@ rl_make_vline_(struct rl_line_s *rrl)
   int i;
   int vposition;
   int key;
-  char *rep;
+  const char *rep;
 
   for (i = vposition = 0; rrl->line[i]; ++i) {
     if (rrl->line[i] != RL_ESC)
@@ -477,10 +465,10 @@ rl_display_line(int clear_to_eol)
   strncpy(line, rl.vline + rl_offset, hx_columns - rl_prompt_len);
   line[hx_columns - rl_prompt_len - 1] = 0;
   if (rl_offset) tio_printf(RL_LEFT_MORE);
-  if (strlen(rl.vline) - rl_offset >= hx_columns - rl_prompt_len)
+  if ((int)(strlen(rl.vline) - rl_offset) >= hx_columns - rl_prompt_len)
     line[hx_columns - rl_prompt_len - 2] = 0;
   tio_raw_printf("%s", line + !!rl_offset);
-  if (rl_get_vlength(&rl) - rl_offset >= hx_columns - rl_prompt_len)
+  if ((int)(rl_get_vlength(&rl) - rl_offset) >= hx_columns - rl_prompt_len)
     tio_printf(RL_RIGHT_MORE);
   tio_goto_column(rl_prompt_len + rl_get_vposition() - rl_offset);
   return 0;
@@ -490,19 +478,19 @@ rl_display_line(int clear_to_eol)
   static int
 rl_insert(int x)
 {
-  int i;
+  size_t i;
   int last_col;
-  int position = rl_get_position();
-  int vposition;
+  size_t position = rl_get_position();
+  size_t vposition;
   int append = (position == strlen(rl.line));
-  char *s = strdup(tio_keyrep(x));
-  int sl = strlen(s);
-  int special_f = 0;
+  char *s = strdup_fatal(tio_keyrep(x));
+  size_t sl = strlen(s);
+  size_t special_f = 0;
   int redisplay_f = 0;
 
   if (!tio_isprint(x) || x == RL_ESC || !x) special_f = 1;
-  for (i = strlen(rl.line) - special_f; i >= position; --i)
-    rl.line[i + 1 + special_f] = rl.line[i];
+  for (i = strlen(rl.line) - special_f + 1; i > position; --i)
+    rl.line[i + special_f] = rl.line[i - 1];
   if (special_f && (x == RL_ESC || x > 0xff || !x)) {
     rl.line[position] = RL_ESC;
     rl.line[position + 1] =
@@ -512,7 +500,7 @@ rl_insert(int x)
   rl_make_vline();
   vposition = rl_get_vposition();
   for (i = 0; i < sl; ++i) {
-    if (vposition - rl_offset + 2 >= hx_columns - rl_prompt_len + append) {
+    if ((ssize_t)(vposition - rl_offset + 2) >= hx_columns - rl_prompt_len + append) {
       ++rl_offset;
       ++vposition;
       tio_goto_column(rl_prompt_len);
@@ -570,12 +558,12 @@ rl_insert(int x)
   static int
 rl_delete(int under_cursor)
 {
-  int i;
+  size_t i;
   int last_col;
-  int special_f = 0;
-  int replen;
-  int position = rl_get_position();
-  int vposition;
+  size_t special_f = 0;
+  size_t replen;
+  size_t position = rl_get_position();
+  size_t vposition;
   int key;
 
   if (!under_cursor && !position) return 0;
@@ -602,24 +590,27 @@ rl_delete(int under_cursor)
       --vposition;
       if (vposition < rl_offset + 1 + !!rl_backspace_jump) {
 	if (rl_backspace_jump && rl_offset) {
-	  rl_offset -= rl_backspace_jump;
+	  ssize_t nofs = rl_offset - rl_backspace_jump;
 	  if (rl_backspace_jump == 1) {
-	    if (rl_offset < 0) {
+	    if (nofs < 0) {
 	      rl_offset = 0;
 	      goto l1;
-	    }
+	    } else
+	      rl_offset = nofs;
 	    tio_goto_column(rl_prompt_len + 1);
 	    tio_putchar(rl.vline[rl_offset + 1]);
 	  } else {
-	    if (rl_offset < 0) rl_offset = 0;
+	    if (nofs < 0)
+	      rl_offset = 0;
+	    else
+	      rl_offset = nofs;
 	    rl_display_line(0);
 	  }
 	} else {
-	  --rl_offset;
-	  if (rl_offset < 0) {
-	    rl_offset = 0;
+	  if (rl_offset == 0)
 	    goto l1;
-	  }
+	  else
+	    --rl_offset;
 	}
       } else {
   l1:   tio_goto_column(rl_prompt_len + vposition - rl_offset);
@@ -696,8 +687,8 @@ rl_ck(void)
   /* clear all up to the end of line.
    */
 {
-  int length = strlen(rl.line);
-  int i;
+  size_t length = strlen(rl.line);
+  size_t i;
 
   for (i = rl_get_position(); i < length; ++i) rl.line[i] = 0;
   rl_make_vline();
@@ -711,9 +702,9 @@ rl_cu(void)
   /* clear all up to the beginning of the line.
    */
 {
-  int position = rl_get_position();
-  int length = strlen(rl.line);
-  int i;
+  size_t position = rl_get_position();
+  size_t length = strlen(rl.line);
+  size_t i;
 
   if (position) {
     for (i = 0; i < length - position; ++i)
@@ -742,15 +733,15 @@ rl_begin(void)
   static int
 rl_end(void)
 {
-  int length;
-  int vposition;
+  size_t length;
+  size_t vposition;
 
   length = rl_get_length(&rl);
   rl_position = length;
   vposition = rl_get_vposition();
-  if (rl_offset < vposition - hx_columns + rl_prompt_len + 1) {
-    rl_offset = vposition - hx_columns + rl_prompt_len + 1;
-    if (rl_offset < 0) rl_offset = 0;
+  ssize_t nofs = vposition - hx_columns + rl_prompt_len + 1;
+  if (nofs > 0 && rl_offset < (size_t)nofs) {
+    rl_offset = nofs;
     rl_display_line(1);
   } else
     tio_goto_column(rl_prompt_len + vposition - rl_offset);
@@ -761,9 +752,9 @@ rl_end(void)
 rl_left(void)
 {
   int last_col;
-  int position;
-  int vposition;
-  int skip;
+  size_t position;
+  size_t vposition;
+  size_t skip;
   int key;
   int redisplay_f = 0;
 
@@ -780,11 +771,10 @@ rl_left(void)
   while (skip--) {
     vposition = rl_get_vposition();
     if (vposition < rl_offset + 1) {
-      --rl_offset;
-      if (rl_offset < 0) {
-	rl_offset = 0;
+      if (rl_offset == 0) {
 	tio_left(1);
       } else {
+	--rl_offset;
 	if (!rl_offset) {
 	  tio_goto_column(rl_prompt_len);
 	  tio_putchar(rl.vline[0]);
@@ -821,9 +811,9 @@ rl_right(void)
   int last_col;
   int append = 0;
   int redisplay_f = 0;
-  int skip;
-  int position;
-  int vposition;
+  size_t skip;
+  size_t position;
+  size_t vposition;
   int key;
 
   if (rl_position == rl_get_length(&rl)) return 0;
@@ -839,7 +829,7 @@ rl_right(void)
   vposition = rl_get_vposition();
   while (skip--) {
     append = (position == strlen(rl.line));
-    if (vposition - rl_offset + 1 >= hx_columns - rl_prompt_len + append) {
+    if ((int)(vposition - rl_offset + 1) >= hx_columns - rl_prompt_len + append) {
       ++rl_offset;
       tio_goto_column(rl_prompt_len);
       if (tio_delete_character()) redisplay_f = 1;
@@ -881,7 +871,7 @@ rl_complete(int context, int again)
   char line[1024];
   char *p;
   int i, j;
-  int prefix_len;
+  size_t prefix_len;
   int stop_f = 0;
 
   if (!completer) return 0;
@@ -941,7 +931,7 @@ rl_complete(int context, int again)
           util_strsort(list);
 	  k = (n - 1) / m + 1; /* number of lines */
 	  /* rearrange the list sorted in hx_columns */
-	  list2 = (char **)malloc(m * k * sizeof(char *));
+	  list2 = (char **)malloc_fatal(m * k * sizeof(char *));
 	  memset(list2, 0, m * k * sizeof(char *));
 	  for (i = 0, j = 0; list[i]; ++i) {
 	    list2[m * (i % k) + j] = list[i];
@@ -966,17 +956,18 @@ dont_list:
 	} else
 	  tio_bell();
       } else {
-	/* comlpete as far as unique */
-	int k = strlen(prefix);
+	/* complete as far as unique */
+	size_t k = strlen(prefix);
 	assert(k > prefix_len);
-	for (i = strlen(rl.line) + (j = k - prefix_len);
-             i >= rl_position;
+	size_t i, j;
+	for (i = strlen(rl.line) + (j = k - prefix_len) + 1;
+             i > rl_position;
              --i)
-	  rl.line[i + j] = rl.line[i];
+	  rl.line[i + j - 1] = rl.line[i - 1];
 	for (i = 0; i < j; ++i)
 	  rl.line[rl_position + i] = prefix[prefix_len + i];
 	rl_position += j;
-	while (rl_position - rl_offset + 1 >= hx_columns - rl_prompt_len)
+	while ((int)(rl_position - rl_offset + 1) >= hx_columns - rl_prompt_len)
 	  ++rl_offset;
 	tio_return();
 	tio_puts(rl_prompt);
@@ -985,16 +976,17 @@ dont_list:
       }
     } else { /* exactly one completion */
       int dir_f;
-      j = strlen(*list) - prefix_len + 1;
+      size_t j = strlen(*list) - prefix_len + 1;
+      size_t i;
       dir_f = (*list)[strlen(*list) - 1] == '/';
       if (dir_f) --j;
-      for (i = strlen(rl.line) + j; i >= rl_position; --i)
-	rl.line[i + j] = rl.line[i];
+      for (i = strlen(rl.line) + j + 1; i > rl_position; --i)
+	rl.line[i + j - 1] = rl.line[i - 1];
       for (i = 0; i < j - !dir_f; ++i)
 	rl.line[rl_position + i] = list[0][prefix_len + i];
       if (!dir_f) rl.line[rl_position + i] = ' ';
       rl_position += j;
-      while (rl_position - rl_offset + 1 >= hx_columns - rl_prompt_len)
+      while ((int)(rl_position - rl_offset + 1) >= hx_columns - rl_prompt_len)
 	++rl_offset;
       tio_return();
       tio_puts(rl_prompt);
@@ -1012,12 +1004,12 @@ dont_list:
 rl_verbatim(void)
 {
   int key;
-  int position = rl_get_position();
+  size_t position = rl_get_position();
   int append = (position == strlen(rl.line));
-  int vposition = rl_get_vposition();
+  size_t vposition = rl_get_vposition();
   int last_col;
 
-  if (vposition - rl_offset + 2 >= hx_columns - rl_prompt_len + append) {
+  if ((int)(vposition - rl_offset + 2) >= hx_columns - rl_prompt_len + append) {
     ++rl_offset;
     tio_goto_column(rl_prompt_len);
     if (tio_delete_character())
@@ -1055,10 +1047,7 @@ restart:
 /* rl_verbatim */
 
   char *
-readline(prompt, default_val, context)
-  char *prompt;
-  char *default_val;
-  int context;
+readline(const char *prompt, const char *default_val, int context)
 {
   int key;
   int escape_f = 0;
@@ -1087,7 +1076,7 @@ readline(prompt, default_val, context)
     rl_default = default_val;
     strncpy(rl.line, default_val, LINE_MAXLEN - 1);
     rl_make_vline();
-    if (rl_prompt_len + rl_position > hx_columns - 2)
+    if ((int)(rl_prompt_len + rl_position) > hx_columns - 2)
       rl_offset = hx_columns - 1 - rl_prompt_len - strlen(rl.vline);
     tio_raw_printf("%s", rl.line + rl_offset);
     tio_goto_column(rl_prompt_len + strlen(rl.vline) - rl_offset);
@@ -1118,7 +1107,7 @@ readline(prompt, default_val, context)
       if (!(hist = rl_history_up())) break;
       strcpy(rl.line, hist->line);
       rl_offset = 0;
-      if (rl_prompt_len + rl_position > hx_columns - 2)
+      if ((int)(rl_prompt_len + rl_position) > hx_columns - 2)
         rl_offset = rl_prompt_len + strlen(rl.vline) - hx_columns + 1;
       rl_display_line(1);
       break;
@@ -1127,7 +1116,7 @@ readline(prompt, default_val, context)
       if (!(hist = rl_history_down())) break;
       strcpy(rl.line, hist->line);
       rl_offset = 0;
-      if (rl_prompt_len + rl_position > hx_columns - 2)
+      if ((int)(rl_prompt_len + rl_position) > hx_columns - 2)
         rl_offset = hx_columns - 2 - rl_prompt_len - strlen(rl.vline);
       rl_display_line(1);
       break;
