@@ -2,7 +2,7 @@
  * Regular expressions, version 0.3
  *
  * This file provides a set of functions for dealing with binary regular
- * expressions, where binary means that some extesions have been made to
+ * expressions, where binary means that some extensions have been made to
  * the standard UN*X-regular expressions making it possible to search for
  * an arbitrary byte pattern.
  *
@@ -16,7 +16,7 @@
  */
 
 /* Copyright (c) 1995,1996 Sascha Demetrio
- * Copyright (c) 2009 - 2011 Peter Pentchev
+ * Copyright (c) 2009 - 2011, 2015 Peter Pentchev
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -24,14 +24,14 @@
  * are met:
  * 1. Redistributions of source code must retain the above copyright
  *    notice, this list of conditions and the following disclaimer.
- *    If you modify any part of HEXER and resitribute it, you must add
+ *    If you modify any part of HEXER and redistribute it, you must add
  *    a notice to the `README' file and the modified source files containing
  *    information about the  changes you made.  I do not want to take
  *    credit or be blamed for your modifications.
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- *    If you modify any part of HEXER and resitribute it in binary form,
+ *    If you modify any part of HEXER and redistribute it in binary form,
  *    you must supply a `README' file containing information about the
  *    changes you made.
  * 3. The name of the developer may not be used to endorse or promote
@@ -63,7 +63,9 @@
 #include <string.h>
 #include <ctype.h>
 
+#include "config.h"
 #include "defs.h"
+#include "util.h"
 
 #undef EOF
 
@@ -162,7 +164,7 @@ enum error_e {
   E_interrupt
 };
 
-char *rx_error_msg[] = {
+const char *rx_error_msg[] = {
   "", /* no error */ "invalid range", "invalid character", "unmatched `('",
   "unmatched `)'", "operator without operand", "malformed operator",
   "operand of `++' and `**' must have fixed length", "unmatched `['",
@@ -212,7 +214,7 @@ static long rx_match_skip;
    */
 
 int rx_nomagic = 0;
-  /* `rx_nomagic == 1': all special characters exept `*' and `[' have to be
+  /* `rx_nomagic == 1': all special characters except `*' and `[' have to be
    *   prefixed with `\'.
    */
 int rx_allmagic = 0;
@@ -227,7 +229,7 @@ long rx_maxmatch = REGEX_MAXMATCH_DFL;
 
 int rx_special_nl = 0;
   /* if `rx_special_nl' is set, the newline character (0xa) is treated as a
-   * special cahacter, i.e. the dot and `ANY_BUT'-ranges ("[^...]") do
+   * special character, i.e. the dot and `ANY_BUT'-ranges ("[^...]") do
    * *not* match a newline character.
    */
 
@@ -361,7 +363,7 @@ regex_store(long slot, long begin, long end)
 }
 /* regex_store */
 
-  static char *
+  static const char *
 regex_ref(int slot)
   /* The contents of undefined slots default to an empty string, therefore
    * the return value is always a valid (non-zero) pointer.
@@ -379,7 +381,7 @@ regex_ref(int slot)
   position = rx_store_begin[slot];
   count = rx_store_end[slot] - position;
   if (!rx_store_match[slot] && count) {
-    rx_store_match[slot] = malloc(count);
+    rx_store_match[slot] = malloc_fatal(count);
     rx_seek(position);
     rx_read(rx_store_match[slot], count);
     return rx_store_match[slot];
@@ -435,7 +437,7 @@ regex_clear(void)
 /* regex_clear */
 
   int
-regex_reset()
+regex_reset(void)
 {
   rx_error = 0;
 
@@ -444,19 +446,8 @@ regex_reset()
 /* regex_reset */
 
   long
-regex_search(regex, begin, end, start, direction,
-             replace_str, replace_len, match_len)
-  long *regex;
-  long begin;
-  long end;
-  long start;
-  int direction; /* >=0: forward; <0: reverse. */
-  char **replace_str;
-  long *replace_len;  /* the replace-string for the match is written to
-		       * `*replace_str'/`replace_len'.  the memory for
-		       * `*replace_str' is allocated by `regex_match()'
-		       * via `malloc()'. */
-  long *match_len;
+regex_search(long *regex, long begin, long end, long start, int direction,
+             char **replace_str, long *replace_len, long *match_len)
 {
   long position;
   int i;
@@ -664,7 +655,7 @@ regex_match_(long *regex, long position, int *parstack, int *parsp)
 	if (max < 0) max = rx_maxmatch;
 	if (max > min) {
 	  long *stack, sp;
-	  stack = (long *)malloc((max - min + 1) * sizeof(long));
+	  stack = (long *)malloc_fatal((max - min + 1) * sizeof(long));
 	  for (sp = 0; max < 0 ? 1 : i < max; ++i) {
 	    if (!regex_match_(pp, p, local_parstack, &local_parsp)) break;
 	    stack[sp++] = p;
@@ -728,18 +719,7 @@ fail:
 /* regex_match_ */
 
   int
-regex_match(regex, position, replace_str, replace_len, match_len)
-  long *regex;        /* pointer to the regex-code to be processed. */
-  long position;      /* position in the input stream. */
-  char **replace_str;
-  long *replace_len;  /* if a match is found at `position',
-                       * the replace-string for the match is written to
-		       * `*replace_str'/`replace_len'.  the memory for
-		       * `*replace_str' is allocated by `regex_match()'
-		       * via `malloc()'. */
-  long *match_len;    /* if a match is found at `position',
-                       * the length of the match is written to
-                       * `*match_len'. */
+regex_match(long *regex, long position, char **replace_str, long *replace_len, long *match_len)
 {
   long *pp = regex + *regex;
   int i;
@@ -748,7 +728,7 @@ regex_match(regex, position, replace_str, replace_len, match_len)
   if (regex_match_(regex + 1, position, 0, &zero)) {
     *match_len = rx_match_skip - position;
     if (!*pp) {
-      *replace_str = (char *)malloc(1);
+      *replace_str = (char *)malloc_fatal(1);
       **replace_str = 0;
       *replace_len = 0;
     } else {
@@ -780,7 +760,7 @@ regex_match(regex, position, replace_str, replace_len, match_len)
       }
       assert(!*pp);
       cp = *replace_str =
-        (char *)malloc(*replace_len + !*replace_len);
+        (char *)malloc_fatal(*replace_len + !*replace_len);
       for (pp = pp_rec;;) {
 	long reflen;
 	if (*pp == (long)EOX) break;
@@ -810,9 +790,7 @@ regex_match(regex, position, replace_str, replace_len, match_len)
 /* regex_match */
 
   long *
-regex_compile(str, replace)
-  char *str;
-  char *replace;
+regex_compile(const char *str, const char *replace)
   /* Compile the regular expression `str' and return a pointer to the
    * compiled regex-code.  If `replace' is non-zero, the string `replace'
    * is interpreted as a replace-string, which may contain `\'-escape
@@ -836,7 +814,8 @@ regex_compile(str, replace)
     if (*++cp == 'x') { ++cp; GET_HEX((n), 32); } else { GET_OCT((n), 32); }  \
   else { GET_DEC((n), 32); } }
 
-  char *cp, *s;
+  const char *cp;
+  char *s;
 
   int pc;
     /* Parentheses counter; number of unmatched parentheses.
@@ -924,10 +903,10 @@ compile:
   if (regex_size) {
     regex_size <<= 1;
     regex =
-      (long *)realloc(regex, regex_size * regex_blocksize * sizeof(long));
+      (long *)realloc_fatal(regex, regex_size * regex_blocksize * sizeof(long));
   } else
     ++regex_size, regex =
-      (long *)malloc(regex_blocksize * sizeof(long));
+      (long *)malloc_fatal(regex_blocksize * sizeof(long));
 
   par[0] = regex + 1;
   branch[0] = regex + 1;
@@ -1231,7 +1210,7 @@ compile:
 	    rx_error = (int)E_unmatched_closing_parenthesis;
 	    break;
 	  }
-	  /* if `i' is set, `j' holds the uniqe length of all branches. */
+	  /* if `i' is set, `j' holds the unique length of all branches. */
 	  if (i) {
 	    BRANCH_LEN_INC(j);
 	  } else {
@@ -1505,7 +1484,7 @@ compile:
    */
   cp = replace;
   escape = 0;
-  s = (char *)malloc(strlen(replace) + 1);
+  s = (char *)malloc_fatal(strlen(replace) + 1);
   j = 0;
   while (*cp) {
     switch (*cp++) {
