@@ -9,7 +9,7 @@
  */
 
 /* Copyright (c) 1995,1996 Sascha Demetrio
- * Copyright (c) 2009 Peter Pentchev
+ * Copyright (c) 2009, 2015 Peter Pentchev
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -17,14 +17,14 @@
  * are met:
  * 1. Redistributions of source code must retain the above copyright
  *    notice, this list of conditions and the following disclaimer.
- *    If you modify any part of HEXER and resitribute it, you must add
+ *    If you modify any part of HEXER and redistribute it, you must add
  *    a notice to the `README' file and the modified source files containing
  *    information about the  changes you made.  I do not want to take
  *    credit or be blamed for your modifications.
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- *    If you modify any part of HEXER and resitribute it in binary form,
+ *    If you modify any part of HEXER and redistribute it in binary form,
  *    you must supply a `README' file containing information about the
  *    changes you made.
  * 3. The name of the developer may not be used to endorse or promote
@@ -94,6 +94,7 @@
 
 #include "defs.h"
 #include "tio.h"
+#include "util.h"
 
 #if USE_SET
 #include "set.h"
@@ -102,7 +103,19 @@
 int tio_readwait_timeout = TIO_READWAIT_TIMEOUT;
 
 #if !HAVE_VASPRINTF
-static char printfbuf[2048];
+static int my_vasprintf(char ** const dst, const char * const fmt, va_list v)
+{
+  char * const p = malloc_fatal(2048);
+  const int n = vsnprintf(p, 2048, fmt, v);
+  if (n < 0 || n >= 2048) {
+    free(p);
+    return -1;
+  }
+  *dst = p;
+  return n;
+}
+#undef vasprintf
+#define vasprintf(dst, fmt, v) my_vasprintf((dst), (fmt), (v))
 #endif
 
 volatile int *tio_interrupt;
@@ -178,24 +191,17 @@ static int		 tio_set_scrolling_region(int first, int last);
 
 #define T_TABSTOP 8
 
-char *terminal_name = 0;
+const char *terminal_name = 0;
 
 int tio_tite_f = 0;
   /* if `tio_tite_f' is set, no "ti"/"te" commands will be sent to the
    * terminal.
    */
 
-#if USE_STDARG
-static void tio_error_msg( const char *fmt, ... );
-static void tio_warning_msg( const char *fmt, ... );
-void (*error_msg)( const char *, ... ) = tio_error_msg;
-void (*warning_msg)( const char *, ... ) = tio_warning_msg;
-#else
-static void tio_error_msg( /* const char *fmt, ... */ );
-static void tio_warning_msg( /* const char *fmt, ... */ );
-void (*error_msg)( /* const char *, ... */ ) = tio_error_msg;
-void (*warning_msg)( /* const char *, ... */ ) = tio_warning_msg;
-#endif
+static void tio_error_msg( const char *fmt, ... ) __printflike(1, 2);
+static void tio_warning_msg( const char *fmt, ... ) __printflike(1, 2);
+void (*error_msg)( const char *, ... ) __printflike(1, 2) = tio_error_msg;
+void (*warning_msg)( const char *, ... ) __printflike(1, 2) = tio_warning_msg;
 void (*tio_winch)(void);
 
 int hx_lines; /* Number of lines. */
@@ -209,122 +215,122 @@ static int t_bg = 0;  /* background color */
 
 static char t_bp[4096];
 static char t_string_buffer[8192];
-static char *program_name;
+static const char *program_name;
 static struct termios ts_start;
 
-char *t_clear_screen;     /* cl */
-char *t_clear_to_eol;     /* ce */
-char *t_clear_to_bol;     /* cb */
-char *t_clear_to_eos;     /* cd */
-char *t_home;             /* ho */
-char *t_last_line;        /* ll */
-char *t_return;           /* cr */
-char *t_reset;            /* rs */
-char *t_bell;             /* bl */
-char *t_visible_bell;     /* vb */
+const char *t_clear_screen;     /* cl */
+const char *t_clear_to_eol;     /* ce */
+const char *t_clear_to_bol;     /* cb */
+const char *t_clear_to_eos;     /* cd */
+const char *t_home;             /* ho */
+const char *t_last_line;        /* ll */
+const char *t_return;           /* cr */
+const char *t_reset;            /* rs */
+const char *t_bell;             /* bl */
+const char *t_visible_bell;     /* vb */
 
-char *t_tc_init;          /* ti */
-char *t_tc_end;           /* te */
-char *t_kp_transmit_on;   /* ks */
-char *t_kp_transmit_off;  /* ke */
+const char *t_tc_init;          /* ti */
+const char *t_tc_end;           /* te */
+const char *t_kp_transmit_on;   /* ks */
+const char *t_kp_transmit_off;  /* ke */
 
-char *t_blink_on;         /* mb */
-char *t_bold_on;          /* md */
-char *t_half_bright_on;   /* mh */
-char *t_reverse_on;       /* mr */
-char *t_standout_on;      /* so */
-char *t_underscore_on;    /* us */
-char *t_underscore_off;   /* ue */
-char *t_all_off;          /* me */
+const char *t_blink_on;         /* mb */
+const char *t_bold_on;          /* md */
+const char *t_half_bright_on;   /* mh */
+const char *t_reverse_on;       /* mr */
+const char *t_standout_on;      /* so */
+const char *t_underscore_on;    /* us */
+const char *t_underscore_off;   /* ue */
+const char *t_all_off;          /* me */
 
-char *t_UP;               /* UP */
-char *t_DOwn;             /* DO */
-char *t_LEft;             /* LE */
-char *t_RIght;            /* RI */
-char *t_up;               /* up */
-char *t_down;             /* do */
-char *t_left;             /* le */
-char *t_right;            /* nd */
-char *t_cm;               /* cm */
-char *t_CM;               /* CM */
-char *t_goto_column;      /* ch */
-char *t_goto_line;        /* cv */
-char *t_save_position;    /* sc */
-char *t_restore_position; /* rc */
-char *t_delete_line;      /* dl */
-char *t_delete_lines;     /* DL */
-char *t_insert_line;      /* al */
-char *t_insert_lines;     /* AL */
-char *t_delete_chars;     /* DC */
-char *t_delete_char;      /* dc */
-char *t_delete_mode;      /* dm */
-char *t_end_delete_mode;  /* ed */
-char *t_insert_chars;     /* IC */
-char *t_insert_char;      /* ic */
-char *t_insert_mode;      /* im */
-char *t_end_insert_mode;  /* ei */
-char *t_scroll_fwd_n;     /* SF (NP*) */
-char *t_scroll_backwd_n;  /* SR (NP*) */
-char *t_scroll_fwd;       /* sf */
-char *t_scroll_backwd;    /* sr */
-char *t_change_scroll;    /* cs (NP) */
+const char *t_UP;               /* UP */
+const char *t_DOwn;             /* DO */
+const char *t_LEft;             /* LE */
+const char *t_RIght;            /* RI */
+const char *t_up;               /* up */
+const char *t_down;             /* do */
+const char *t_left;             /* le */
+const char *t_right;            /* nd */
+const char *t_cm;               /* cm */
+const char *t_CM;               /* CM */
+const char *t_goto_column;      /* ch */
+const char *t_goto_line;        /* cv */
+const char *t_save_position;    /* sc */
+const char *t_restore_position; /* rc */
+const char *t_delete_line;      /* dl */
+const char *t_delete_lines;     /* DL */
+const char *t_insert_line;      /* al */
+const char *t_insert_lines;     /* AL */
+const char *t_delete_chars;     /* DC */
+const char *t_delete_char;      /* dc */
+const char *t_delete_mode;      /* dm */
+const char *t_end_delete_mode;  /* ed */
+const char *t_insert_chars;     /* IC */
+const char *t_insert_char;      /* ic */
+const char *t_insert_mode;      /* im */
+const char *t_end_insert_mode;  /* ei */
+const char *t_scroll_fwd_n;     /* SF (NP*) */
+const char *t_scroll_backwd_n;  /* SR (NP*) */
+const char *t_scroll_fwd;       /* sf */
+const char *t_scroll_backwd;    /* sr */
+const char *t_change_scroll;    /* cs (NP) */
 
-char *t_standout_cursor;  /* vs */
-char *t_normal_cursor;    /* ve */
-char *t_invisible_cursor; /* vi */
+const char *t_standout_cursor;  /* vs */
+const char *t_normal_cursor;    /* ve */
+const char *t_invisible_cursor; /* vi */
 
-char *t_key_delete;       /* kD */
-char *t_key_backspace;    /* kb */
-char *t_key_tab;          /* kT */
-char *t_key_return = "\n";
-char *t_key_tilde = "~";
-char *t_key_hat = "^";
-char *t_key_null = "";
-char *t_key_break = "\3";
+const char *t_key_delete;       /* kD */
+const char *t_key_backspace;    /* kb */
+const char *t_key_tab;          /* kT */
+const char *t_key_return = "\n";
+const char *t_key_tilde = "~";
+const char *t_key_hat = "^";
+const char *t_key_null = "";
+const char *t_key_break = "\3";
 
-char *t_key_up;           /* ku */
-char *t_key_down;         /* kd */
-char *t_key_left;         /* kl */
-char *t_key_right;        /* kr */
-char *t_key_f0;           /* k0 */
-char *t_key_f1;           /* k1 */
-char *t_key_f2;           /* k2 */
-char *t_key_f3;           /* k3 */
-char *t_key_f4;           /* k4 */
-char *t_key_f5;           /* k5 */
-char *t_key_f6;           /* k6 */
-char *t_key_f7;           /* k7 */
-char *t_key_f8;           /* k8 */
-char *t_key_f9;           /* k9 */
-char *t_key_f10;          /* k; */
-char *t_key_backtab;      /* kB */
-char *t_key_begin;        /* @1 */
-char *t_key_cancel;       /* @2 */
-char *t_key_close;        /* @3 */
-char *t_key_command;      /* @4 */
-char *t_key_copy;         /* @5 */
-char *t_key_create;       /* @6 */
-char *t_key_end;          /* @7 */
-char *t_key_enter;        /* @8 */
-char *t_key_exit;         /* @9 */
-char *t_key_upper_left;   /* K1 */
-char *t_key_upper_right;  /* K3 */
-char *t_key_center;       /* K2 */
-char *t_key_bottom_left;  /* K4 */
-char *t_key_bottom_right; /* K5 */
-char *t_key_home;         /* ho */
-char *t_key_page_up;      /* kP */
-char *t_key_page_down;    /* kN */
+const char *t_key_up;           /* ku */
+const char *t_key_down;         /* kd */
+const char *t_key_left;         /* kl */
+const char *t_key_right;        /* kr */
+const char *t_key_f0;           /* k0 */
+const char *t_key_f1;           /* k1 */
+const char *t_key_f2;           /* k2 */
+const char *t_key_f3;           /* k3 */
+const char *t_key_f4;           /* k4 */
+const char *t_key_f5;           /* k5 */
+const char *t_key_f6;           /* k6 */
+const char *t_key_f7;           /* k7 */
+const char *t_key_f8;           /* k8 */
+const char *t_key_f9;           /* k9 */
+const char *t_key_f10;          /* k; */
+const char *t_key_backtab;      /* kB */
+const char *t_key_begin;        /* @1 */
+const char *t_key_cancel;       /* @2 */
+const char *t_key_close;        /* @3 */
+const char *t_key_command;      /* @4 */
+const char *t_key_copy;         /* @5 */
+const char *t_key_create;       /* @6 */
+const char *t_key_end;          /* @7 */
+const char *t_key_enter;        /* @8 */
+const char *t_key_exit;         /* @9 */
+const char *t_key_upper_left;   /* K1 */
+const char *t_key_upper_right;  /* K3 */
+const char *t_key_center;       /* K2 */
+const char *t_key_bottom_left;  /* K4 */
+const char *t_key_bottom_right; /* K5 */
+const char *t_key_home;         /* ho */
+const char *t_key_page_up;      /* kP */
+const char *t_key_page_down;    /* kN */
 
 /* terminfo variables
  */
-char *t_setab;            /* AB */
-char *t_setaf;            /* AF */
+const char *t_setab;            /* AB */
+const char *t_setaf;            /* AF */
 
 
 static struct t_strings_s {
-  char *id;
-  char **string;
+  const char *id;
+  const char **string;
 } t_strings[] = {
   /* Codes sent by special keys.
    */
@@ -436,12 +442,12 @@ static struct t_strings_s {
 };
 
 static struct t_require_s {
-  char *required[T_REQ_MAX_CAPS];
-    /* A null-terminated list of string capabisities.  At least one of the
+  const char *required[T_REQ_MAX_CAPS];
+    /* A null-terminated list of string capabilities.  At least one of the
      * capabilities in the list has to be present in the termcap entry.
      * At most `T_REQ_MAX_CAPS' capabilities may be listed.
      */
-  char *message;
+  const char *message;
     /* Message to be printed in case the requirement isn't met.  The
      * `message' may contain one or more `%s'.  These are replaced by the
      * capabilities in the `required' list.  At most 8 `%s' may appear in
@@ -468,9 +474,9 @@ static struct t_require_s {
 
 static struct t_keys_s {
   enum t_keys_e key; /* Internal representation for the key. */
-  char **string;     /* String sent by the terminal. */
-  char *name;        /* Name of key. */
-  char *srep;        /* String representation of the key (i.e. ~1F for F1) */
+  const char **string;     /* String sent by the terminal. */
+  const char *name;        /* Name of key. */
+  const char *srep;        /* String representation of the key (i.e. ~1F for F1) */
 } t_keys[] = {
   { HXKEY_DELETE, &t_key_delete, "delete", "~DEL" },
   { HXKEY_BACKSPACE, &t_key_backspace, "backspace", "~BS" },
@@ -512,7 +518,7 @@ static struct t_keys_s {
   { (enum t_keys_e)'^', &t_key_hat, "hat", "~HAT" },
   { HXKEY_NULL, &t_key_null, "null", "^@" },
   { HXKEY_BREAK, &t_key_break, "break", "~BREAK" },
-  { (enum t_keys_e)0, 0 }
+  { (enum t_keys_e)0, 0, 0, 0 }
 };
 #define HXKEY_ESCAPE_NAME "escape"
 
@@ -523,8 +529,7 @@ static struct t_keys_s {
    */
 
   int
-tio_isprint(x)
-  int x;
+tio_isprint(int x)
 {
   if (x < 0) x += 0x100;
   if (x < 0x20) return 0;
@@ -545,23 +550,12 @@ tio_isprint(x)
 }
 /* tio_isprint */
 
-#if USE_STDARG
   static void
-tio_error_msg(const char *fmt, ...)
-#else
-  static void
-tio_error_msg(fmt, va_alist)
-  const char *fmt;
-  va_dcl
-#endif
+tio_error_msg(const char * const fmt, ...)
 {
   va_list ap;
 
-#if USE_STDARG
   va_start(ap, fmt);
-#else
-  va_start(ap);
-#endif
   fprintf(stderr, "%s: ", program_name);
   vfprintf(stderr, fmt, ap);
   fprintf(stderr, "\n");
@@ -569,23 +563,12 @@ tio_error_msg(fmt, va_alist)
 }
 /* tio_error_msg */
 
-#if USE_STDARG
   static void
-tio_warning_msg(const char *fmt, ...)
-#else
-  static void
-tio_warning_msg(fmt, va_alist)
-  const char *fmt;
-  va_dcl
-#endif
+tio_warning_msg(const char * const fmt, ...)
 {
   va_list ap;
 
-#if USE_STDARG
   va_start(ap, fmt);
-#else
-  va_start(ap);
-#endif
   fprintf(stderr, "%s: warning: ", program_name);
   vfprintf(stderr, fmt, ap);
   fprintf(stderr, "\n");
@@ -593,7 +576,7 @@ tio_warning_msg(fmt, va_alist)
 }
 /* tio_warning_msg */
 
-  static char *
+  static const char *
 tio_cap(const char *id)
   /* Returns the string for capability `id'.  This function woks only
    * for capabilities that have already been read from the termcap entry.
@@ -627,16 +610,8 @@ tio_listcaps(void)
 /* tio_listcaps */
 #endif
 
-#if USE_STDARG
   static int
-tio_command(char *cmd, int affcnt, ...)
-#else
-  static int
-tio_command(cmd, affcnt, va_alist)
-  char *cmd;
-  int affcnt;
-  va_dcl
-#endif
+tio_command(const char * const cmd, const int affcnt, ...)
   /* send the command `cmd' to the terminal.
    * return value: 0 if all goes well and -1 on error.
    */
@@ -645,11 +620,7 @@ tio_command(cmd, affcnt, va_alist)
   int arg1, arg2;
 
   if (!cmd) return -1;
-#if USE_STDARG
   va_start(ap, affcnt);
-#else
-  va_start(ap);
-#endif
   arg1 = va_arg(ap, int);
   arg2 = va_arg(ap, int);
   tputs(tgoto(cmd, arg2, arg1), affcnt, outc);
@@ -658,8 +629,7 @@ tio_command(cmd, affcnt, va_alist)
 /* tio_command */
 
   int
-tio_init(prog)
-  char *prog;
+tio_init(const char * const prog)
   /* Initialize.  This function should be called before any other
    * `tio_*'-function.  `prog' should be the name of the application.
    * If you call `tio_init()' with `prog == 0', the program name is set
@@ -708,7 +678,7 @@ tio_init(prog)
      * termcap entry, use ASCII-codes.
      */
   for (i = 0; *t_required[i].required; ++i) {
-    char **p;
+    const char **p;
     int met;
     for (p = t_required[i].required, met = 0; *p; ++p)
       if (tio_cap(*p)) { met = 1; break; }
@@ -816,7 +786,7 @@ tio_init(prog)
 #ifdef SA_INTERRUPT
     static struct sigaction act;
     act.sa_handler = sigwinch_handler;
-    act.sa_mask = 0;
+    sigemptyset(&act.sa_mask);
     act.sa_flags = SA_INTERRUPT;
     sigaction(SIGWINCH, &act, 0);
 #else
@@ -835,7 +805,7 @@ fail:
 /* tio_init */
 
   void
-tio_return()
+tio_return(void)
 {
   tio_command(t_return, 1);
   t_column = 0;
@@ -843,10 +813,9 @@ tio_return()
 /* tio_return */
 
   void
-tio_up(count)
+tio_up(int count)
   /* Move cursor up `count' lines.
    */
-  int count;
 {
   if (!count) return;
   if (t_line < 0)
@@ -864,10 +833,9 @@ tio_up(count)
 /* tio_up */
 
   void
-tio_down(count)
+tio_down(int count)
   /* Move cursor down `count' lines.
    */
-  int count;
 {
   if (!count) return;
   if (t_line < 0) {
@@ -888,10 +856,9 @@ tio_down(count)
 /* tio_down */
 
   void
-tio_left(count)
+tio_left(int count)
   /* Move cursor left `count' lines.
    */
-  int count;
 {
   if (!count) return;
   if (t_line < 0)
@@ -911,10 +878,9 @@ tio_left(count)
 /* tio_left */
 
   void
-tio_right(count)
+tio_right(int count)
   /* Move cursor right `count' lines.
    */
-  int count;
 {
   if (!count) return;
   if (t_column < 0) {
@@ -935,7 +901,7 @@ tio_right(count)
 /* tio_right */
 
   void
-tio_start_application()
+tio_start_application(void)
 {
   if (!tio_tite_f) tio_command(t_tc_init, 1);
   t_application_mode = 1;
@@ -943,7 +909,7 @@ tio_start_application()
 /* tio_start_application */
 
   void
-tio_end_application()
+tio_end_application(void)
 {
   if (!tio_tite_f) tio_command(t_tc_end, 1);
   t_application_mode = 0;
@@ -951,8 +917,7 @@ tio_end_application()
 /* tio_end_application */
 
   void
-tio_keypad(on)
-  int on;
+tio_keypad(const int on)
   /* Set the keypad mode.
    * on=0: numeric keypad.
    * on=1: application keypad.  Select this mode if you want to use the
@@ -971,7 +936,7 @@ tio_keypad(on)
 /* tio_keypad */
 
 /* The functions `tio_read()' and `tio_unread()' are an interface to the
- * standard input stream (stdin).  It is possibe to unread an arbitrary
+ * standard input stream (stdin).  It is possible to unread an arbitrary
  * number of characters (at most 4096).  `tio_read()' and `tio_unread()'
  * are called by `tio_getch_()'.
  */
@@ -999,14 +964,14 @@ tio_unread(char *buf, long count)
 /* tio_unread */
 
   int
-tio_readmore()
+tio_readmore(void)
 {
   return tio_unread_count;
 }
 /* tio_readmore */
 
   int
-tio_getmore()
+tio_getmore(void)
 {
   return tio_unget_count;
 }
@@ -1040,12 +1005,12 @@ tio_getch_(void)
   tio_unget_count = 0;
 
   /* Try to read one character from the input stream.  If the terminal is not
-   * in keypad transmit mode, the caracter is returned.
+   * in keypad transmit mode, the character is returned.
    */
   switch (tio_read(cbuf, 1)) {
   case -1:
     return (int)HXKEY_ERROR;
-      /* The occurence of an error in `tio_read()' is likely to be caused by
+      /* The occurrence of an error in `tio_read()' is likely to be caused by
        * the signal `SIGWINCH', indicating that the size of the window has
        * changed.
        */
@@ -1080,7 +1045,7 @@ tio_getch_(void)
   }
 
   /* If the read character is a prefix of a known key string, characters
-   * are read from the input stream until a key string is recoginzed,
+   * are read from the input stream until a key string is recognized,
    * the read string is no longer a prefix of a known key string or no
    * characters arrived for `t_timeout' tenths of a second.
    */
@@ -1120,7 +1085,7 @@ tio_getch_(void)
 /* tio_getch_ */
 
   int
-tio_getch()
+tio_getch(void)
   /* Read a character or keypad key from the keyboard.  `tio_getch()' waits
    * for input and returns the key pressed or `HXKEY_ERROR' (-1) on error.
    * This function calls `tio_getch_()' to read the character.
@@ -1158,7 +1123,7 @@ tio_getch()
 /* tio_getch */
 
   int
-tio_get()
+tio_get(void)
   /* Like `tio_getch()', but doesn't wait input.  `tio_get()' returns
    * `HXKEY_NONE' (0) if no input is available.
    */
@@ -1213,12 +1178,11 @@ tio_tget(int tmout)
 /* tio_tget */
 
   int
-tio_ungetch(x)
+tio_ungetch(int x)
   /* Put the character `x' back into the input stream.  At most
    * `TIO_MAX_UNGET' characters can be ungetch.  The return value is `x'
    * or -1 on error.
    */
-  int x;
 {
   if (tio_unget_count >= TIO_MAX_UNGET) return -1;
   tio_unget_buffer[tio_unget_count++] = x;
@@ -1227,11 +1191,10 @@ tio_ungetch(x)
 /* tio_ungetch */
 
   int
-tio_ungets(x)
+tio_ungets(int *x)
   /* Put the key string `x' back into the input stream using
    * `tio_ungetch()'.  The return value is 0 and -1 on error.
    */
-  int *x;
 {
   int i;
 
@@ -1242,12 +1205,11 @@ tio_ungets(x)
 /* tio_ungets */
 
   int
-tio_testkey(key)
+tio_testkey(int key)
   /* Returns 1, if a termcap entry for the requested key exists, else 0.
    * The function return always 1 for the keys `HXKEY_BACKSPACE', `HXKEY_TAB',
    * `HXKEY_RETURN', `HXKEY_ESCAPE', `HXKEY_DELETE', `HXKEY_NONE' and `HXKEY_ERROR'.
    */
-  int key;
 {
   int i;
 
@@ -1258,14 +1220,13 @@ tio_testkey(key)
 }
 /* tio_testkey */
 
-  char *
-tio_keyname(key)
+  const char *
+tio_keyname(int key)
   /* Returns the name of the key `key'.  If `key' is a printable character,
    * it is returned as a string.  If `key' is a special key, the name of
    * that key is returned.  If `key' is unknown and greater than 0xff "??"
    * is returned, else a `\x??' hexadecimal code.
    */
-  int key;
 {
   int i;
   static char name[8];
@@ -1284,9 +1245,8 @@ tio_keyname(key)
 }
 /* tio_keyname */
 
-  char *
-tio_keyrep(key)
-  int key;
+  const char *
+tio_keyrep(int key)
 {
   static char rep[8];
   int i;
@@ -1310,9 +1270,8 @@ tio_keyrep(key)
 }
 /* tio_keyrep */
 
-  char *
-tio_vkeyrep(key)
-  int key;
+  const char *
+tio_vkeyrep(int key)
 {
   int i;
 
@@ -1323,10 +1282,7 @@ tio_vkeyrep(key)
 /* tio_vkeyrep */
 
   char *
-tio_keyscan(key, s, mode)
-  int *key;
-  char *s;
-  int mode;
+tio_keyscan(int *key, char *s, int mode)
   /* Check if `s' is a sting representation of a key.
    * the keycode is written to `*key' and a pointer to the first
    * character after the srep is returned.
@@ -1356,15 +1312,14 @@ tio_keyscan(key, s, mode)
 /* tio_keyscan */
 
   int
-tio_echo(on)
-  int on;
+tio_echo(const int on)
   /* on=1:  characters are echoed as they are typed.
    * on=0:  characters are not echoed.
-   * Thie echo-option has no effect on other `tio_*'-functions,
+   * The echo-option has no effect on other `tio_*'-functions,
    * i.e. `tio_getch()' won't ever echo the character it reads.
    * RETURN VALUE:  `tio_echo()' returns the previous echo-state; that is
    *   1 if echo was set and 0 if echo was clear.
-   * NOTE:  You may want to turn of the echo if your program is makeing
+   * NOTE:  You may want to turn of the echo if your program is making
    *   several successive calls to `tio_getch()', so the characters typed
    *   between two calls to `tio_getch()' won't be echoed on the screen.
    */
@@ -1384,7 +1339,7 @@ tio_echo(on)
 /* tio_echo */
 
   void
-tio_home()
+tio_home(void)
   /* Cursor home. */
 {
   if (t_home)
@@ -1399,7 +1354,7 @@ tio_home()
 /* tio_home */
 
   void
-tio_last_line()
+tio_last_line(void)
   /* Move the cursor to the last line, first column.  */
 {
   if (t_last_line) {
@@ -1412,14 +1367,13 @@ tio_last_line()
 /* tio_last_line */
 
   void
-tio_goto_line(line)
-  int line;
+tio_goto_line(const int p_line)
   /* Move cursor to line `line'.
    */
 {
   char *cmd;
 
-  if (line > hx_lines - 1) line = hx_lines - 1;
+  const int line = p_line > hx_lines - 1? hx_lines - 1: p_line;
   if (t_goto_line)
     tio_command(t_goto_line, 1, line);
   else {
@@ -1432,8 +1386,7 @@ tio_goto_line(line)
 /* tio_goto_line */
 
   void
-tio_goto_column(column)
-  int column;
+tio_goto_column(int column)
   /* Move cursor to column `column'.
    */
 {
@@ -1454,8 +1407,7 @@ tio_goto_column(column)
 /* tio_goto_column */
 
   void
-tio_move(lin, col)
-  int lin, col;
+tio_move(int lin, int col)
   /* Move the cursor to position `line'/`column'.
    */
 {
@@ -1471,8 +1423,7 @@ tio_move(lin, col)
 /* tio_move */
 
   void
-tio_rel_move(lin, col)
-  int lin, col;
+tio_rel_move(int lin, int col)
   /* Move the cursor relative to the cursor position.
    */
 {
@@ -1482,8 +1433,7 @@ tio_rel_move(lin, col)
 /* tio_rel_move */
 
   static int
-tio_set_scrolling_region(first, last)
-  int first, last;
+tio_set_scrolling_region(int first, int last)
 {
   if (t_change_scroll) {
     if (last < 0) last = hx_lines - 1;
@@ -1502,9 +1452,7 @@ tio_reset_scrolling_region(void)
 /* tio_reset_scrolling_region */
 
   int
-tio_scroll_up(count, first, last)
-  int count;
-  int first, last;
+tio_scroll_up(int count, int first, int last)
   /* Scroll up (forward) `count' lines.  returns -1 if the terminal
    * can't scroll.
    */
@@ -1559,9 +1507,7 @@ scroll_all:
 /* tio_scroll_up */
 
   int
-tio_scroll_down(count, first, last)
-  int count;
-  int first, last;
+tio_scroll_down(int count, int first, int last)
   /* Scroll down (reverse) `count' lines.  returns -1 if the terminal can't
    * scroll backwards.
    */
@@ -1606,19 +1552,19 @@ tio_scroll_down(count, first, last)
 /* tio_scroll_down */
 
   int
-tio_puts(s)
-  char *s;
+tio_puts(const char *s)
   /* Like `fputs(s, stdout)'.
    */
 {
-  char *c1, *c2;
+  const char *c1;
+  char *c2;
   char *out;
   int mask_f, x;
 
   if (t_line < 0) tio_goto_line(0);
   if (t_column < 0) tio_goto_column(0);
   c1 = s;
-  c2 = out = (char *)malloc(strlen(s) + 1);
+  c2 = out = (char *)malloc_fatal(strlen(s) + 1);
   for (; *c1; ++c1) {
     mask_f = 0;
     switch (*c1) {
@@ -1659,8 +1605,7 @@ tio_puts(s)
 /* tio_puts */
 
   int
-tio_putchar(x)
-  int x;
+tio_putchar(int x)
   /* Like `putchar(x)'.
    */
 {
@@ -1674,28 +1619,28 @@ tio_putchar(x)
 /* tio_putchar */
 
   void
-tio_bell()
+tio_bell(void)
 {
   if (t_bell) tio_command(t_bell, 1); else putchar('\a');
 }
 /* tio_bell */
 
   void
-tio_visible_bell()
+tio_visible_bell(void)
 {
   tio_command(t_visible_bell, 1);
 }
 /* tio_visible_bell */
 
   void
-tio_underscore()
+tio_underscore(void)
 {
   tio_command(t_underscore_on, 1);
 }
 /* tio_underscore */
 
   void
-tio_underscore_off()
+tio_underscore_off(void)
 {
   tio_command(t_underscore_off, 1);
   tio_set_colors(t_fg, t_bg);
@@ -1703,35 +1648,35 @@ tio_underscore_off()
 /* tio_underscore_off */
 
   void
-tio_bold()
+tio_bold(void)
 {
   tio_command(t_bold_on, 1);
 }
 /* tio_bold */
 
   void
-tio_blink()
+tio_blink(void)
 {
   tio_command(t_blink_on, 1);
 }
 /* tio_blink */
 
   void
-tio_half_bright()
+tio_half_bright(void)
 {
   tio_command(t_half_bright_on, 1);
 }
 /* tio_half_bright */
 
   void
-tio_reverse()
+tio_reverse(void)
 {
   tio_command(t_reverse_on, 1);
 }
 /* tio_reverse */
 
   void
-tio_normal()
+tio_normal(void)
 {
   tio_command(t_all_off, 1);
   tio_command(t_underscore_off, 1);
@@ -1740,13 +1685,20 @@ tio_normal()
 /* tio_normal */
 
   void
-tio_set_cursor(mode)
+tio_reset_attributes(void)
+{
+  tio_command(t_all_off, 1);
+  tio_clear_to_eol();
+}
+/* tio_reset_attributes */
+
+  void
+tio_set_cursor(int mode)
   /* Set the visibility of the cursor.
    * `mode == 0':  invisible.
    * `mode == 1':  normal.
    * `mode == 2':  standout.
    */
-  int mode;
 {
   switch (mode) {
   case 0:
@@ -1763,7 +1715,7 @@ tio_set_cursor(mode)
 /* tio_set_cursor */
 
   int
-tio_have_color()
+tio_have_color(void)
   /* returns a non-zero value if color is available, 0 else.
    */
 {
@@ -1776,7 +1728,7 @@ tio_have_color()
 /* tio_have_color */
 
   void
-tio_set_colors(fg, bg)
+tio_set_colors(int fg, int bg)
 {
   tio_set_fg(fg);
   tio_set_bg(bg);
@@ -1784,8 +1736,7 @@ tio_set_colors(fg, bg)
 /* tio_set_colors */
 
   void
-tio_set_fg(color)
-  int color;
+tio_set_fg(int color)
   /* set the foreground color to `color'.
    */
 {
@@ -1797,8 +1748,7 @@ tio_set_fg(color)
 /* tio_set_af */
 
   void
-tio_set_bg(color)
-  int color;
+tio_set_bg(int color)
   /* set the background color to `color'.
    */
 {
@@ -1810,21 +1760,21 @@ tio_set_bg(color)
 /* tio_set_ab */
 
   int
-tio_get_fg()
+tio_get_fg(void)
 {
   return t_fg;
 }
 /* tio_get_af */
 
   int
-tio_get_bg()
+tio_get_bg(void)
 {
   return t_bg;
 }
 /* tio_get_bg */
 
   void
-tio_clear()
+tio_clear(void)
 {
   tio_command(t_clear_screen, 1);
   t_line = 0;
@@ -1833,14 +1783,14 @@ tio_clear()
 /* tio_clear */
 
   void
-tio_clear_to_eol()
+tio_clear_to_eol(void)
 {
   tio_command(t_clear_to_eol, 1);
 }
 /* clear_to_eol */
 
   void
-tio_display(text, indent_arg)
+tio_display(char *text, int indent_arg)
   /* Send the string `text' to the terminal.  The string may contain the
    * following `@*'-commands:
    *
@@ -1872,13 +1822,11 @@ tio_display(text, indent_arg)
    * `indent_arg' sets the indent.  If `indent_arg < 0', the indent stays
    * unchanged.
    */
-  char *text;
-  int indent_arg;
 {
   int i, j;
   int lin, col;
   int eol_f, back_f, absolute_f;
-  char *s = (char *)malloc(strlen(text) + 1);
+  char *s = (char *)malloc_fatal(strlen(text) + 1);
   static int indent = 0;
   int maxcol = hx_columns - 1;
 
@@ -1968,9 +1916,7 @@ tio_display(text, indent_arg)
 /* tio_display */
 
   void
-tio_message(message, indent)
-  char **message;
-  int indent;
+tio_message(char **message, int indent)
   /* displays the array of strings `message' via `tio_display()' providing
    * a simple pager.  this function assumes, that every string in
    * `message' displays as a single line.
@@ -2010,24 +1956,13 @@ tio_message(message, indent)
 }
 /* tio_message */
 
-#if USE_STDARG
   int
-tio_printf(const char *fmt, ...)
-#else
-  int
-tio_printf(fmt, va_alist)
-  const char *fmt;
-  va_dcl
-#endif
+tio_printf(const char * const fmt, ...)
 {
   va_list ap;
   int rval;
 
-#if USE_STDARG
   va_start(ap, fmt);
-#else
-  va_start(ap);
-#endif
   rval = tio_vprintf(fmt, ap);
   va_end(ap);
   return rval;
@@ -2035,9 +1970,7 @@ tio_printf(fmt, va_alist)
 /* tio_printf */
 
   int
-tio_vprintf(fmt, ap)
-  const char *fmt;
-  va_list ap;
+tio_vprintf(const char * const fmt, va_list ap)
   /* Similar to `printf()'.  `tio_printf()' understands the same @-commands
    * as `tio_display()'.  Note that @-commands in strings inserted via %s
    * are executed.  Use `tio_raw_printf()' if you don't wan't @-commands to
@@ -2047,40 +1980,22 @@ tio_vprintf(fmt, ap)
   char *s;
   int rval;
 
-#if HAVE_VASPRINTF
   rval = vasprintf(&s, fmt, ap);
-#else
-  rval = vsnprintf(printfbuf, sizeof(printfbuf), fmt, ap);
-  s = printfbuf;
-#endif
   if (rval == -1)
     return rval;
   tio_display(s, 0);
-#if HAVE_VASPRINTF
   free((char *)s);
-#endif
   return rval;
 }
 /* tio_vprintf */
 
-#if USE_STDARG
   int
 tio_raw_printf(const char *fmt, ...)
-#else
-  int
-tio_raw_printf(fmt, va_alist)
-  const char *fmt;
-  va_dcl
-#endif
 {
   va_list ap;
   int rval;
 
-#if USE_STDARG
   va_start(ap, fmt);
-#else
-  va_start(ap);
-#endif
   rval = tio_raw_vprintf(fmt, ap);
   va_end(ap);
   return rval;
@@ -2088,34 +2003,24 @@ tio_raw_printf(fmt, va_alist)
 /* tio_raw_printf */
 
   int
-tio_raw_vprintf(fmt, ap)
-  const char *fmt;
-  va_list ap;
+tio_raw_vprintf(const char *fmt, va_list ap)
   /* Like `printf()'.  No @-commands.
    */
 {
   char *s;
   int rval;
 
-#if HAVE_VASPRINTF
   rval = vasprintf(&s, fmt, ap);
-#else
-  rval = snprintf(printfbuf, sizeof(printfbuf), fmt, ap);
-  s = printfbuf;
-#endif
   if (rval == -1)
     return rval;
   tio_puts(s);
-#if HAVE_VASPRINTF
   free((char *)s);
-#endif
   return rval;
 }
 /* tio_raw_vprintf */
 
   int
-tio_insert_character(x)
-  char x;
+tio_insert_character(char x)
   /* Insert the character `x' at the current position.
    */
 {
@@ -2138,7 +2043,7 @@ tio_insert_character(x)
 /* tio_insert_character */
 
   int
-tio_delete_character()
+tio_delete_character(void)
   /* Delete the character under the cursor.
    */
 {
@@ -2154,7 +2059,7 @@ tio_delete_character()
 
 #ifdef SIGWINCH
   static sigtype_t
-sigwinch_handler(int sig)
+sigwinch_handler(int sig __unused)
   /* Signal handler for signal `SIGWINCH'.
    */
 {
@@ -2174,7 +2079,7 @@ sigwinch_handler(int sig)
 #endif
 
   int
-tio_raw_readwait(tmout)
+tio_raw_readwait(int tmout)
 {
 #ifdef FD_ZERO
   fd_set fdset;
@@ -2224,14 +2129,14 @@ tio_getwait(int tmout)
 /* tio_getwait */
 
   void
-tio_reset()
+tio_reset(void)
 {
   tcsetattr(0, TCSANOW, &ts_start);
 }
 /* tio_reset */
 
   void
-tio_suspend()
+tio_suspend(void)
 {
   if ((t_keypad_transmit_recover = t_keypad_transmit))
     tio_keypad(0);
@@ -2243,12 +2148,13 @@ tio_suspend()
 /* tio_suspend */
 
   void
-tio_restart()
+tio_restart(void)
 {
   if (t_keypad_transmit_recover) tio_keypad(1);
   if (t_application_mode_recover) tio_start_application();
   tcsetattr(0, TCSANOW, &ts_recover);
   tio_init(0);
+  tio_normal();
 }
 /* tio_restart */
 
